{
    "$graph": [
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "label": "Input BAM file",
                    "doc": "Input BAM file, sorted by coordinates",
                    "id": "#bam-bedgraph-bigwig.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Output filename for generated bedGraph",
                    "id": "#bam-bedgraph-bigwig.cwl/bedgraph_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Output filename for generated bigWig",
                    "id": "#bam-bedgraph-bigwig.cwl/bigwig_filename"
                },
                {
                    "type": "File",
                    "doc": "Tab delimited chromosome length file: <chromName><TAB><chromSize>",
                    "id": "#bam-bedgraph-bigwig.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Change strand af the mate read, so both reads come from the same strand",
                    "id": "#bam-bedgraph-bigwig.cwl/dutp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Set fixed fragment size for genome coverage calculation",
                    "id": "#bam-bedgraph-bigwig.cwl/fragment_size"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Parameter to calculate scale as 1000000/mapped_reads_number. Ignored by bedtools-genomecov.cwl in\nbam_to_bedgraph step if scale is provided\n",
                    "id": "#bam-bedgraph-bigwig.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Enable paired-end genome coverage calculation",
                    "id": "#bam-bedgraph-bigwig.cwl/pairchip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "doc": "Coefficient to scale the genome coverage by a constant factor",
                    "id": "#bam-bedgraph-bigwig.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Calculate genome coverage for each part of the splitted by 'N' and 'D' read",
                    "id": "#bam-bedgraph-bigwig.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Calculate genome coverage of intervals from a specific strand",
                    "id": "#bam-bedgraph-bigwig.cwl/strand"
                }
            ],
            "steps": [
                {
                    "run": "#bedtools-genomecov.cwl",
                    "in": [
                        {
                            "default": "-bg",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/depth"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/dutp",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/du"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/fragment_size",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/fragment_size"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bam_file",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/input_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/mapped_reads_number",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/mapped_reads_number"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bedgraph_filename",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/output_filename"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/pairchip",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/pairchip"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/scale",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/scale"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/split",
                            "valueFrom": "${\n  if (self == null){\n    return true;\n  } else {\n    return self;\n  }\n}\n",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/split"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/strand",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/strand"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/genome_coverage_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph"
                },
                {
                    "run": "#linux-sort.cwl",
                    "in": [
                        {
                            "default": [
                                "1,1",
                                "2,2n"
                            ],
                            "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/key"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/genome_coverage_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/unsorted_file"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph"
                },
                {
                    "run": "#ucsc-bedgraphtobigwig.cwl",
                    "in": [
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bedgraph_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/chrom_length_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/chrom_length_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bigwig_filename",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/output_filename"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bigwig_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig"
                }
            ],
            "doc": "Workflow converts input BAM file into bigWig and bedGraph files.\n\nInput BAM file should be sorted by coordinates (required by `bam_to_bedgraph` step).\n\nIf `split` input is not provided use true by default. Default logic is implemented in `valueFrom` field of `split`\ninput inside `bam_to_bedgraph` step to avoid possible bug in cwltool with setting default values for workflow inputs.\n\n`scale` has higher priority over the `mapped_reads_number`. The last one is used to calculate `-scale` parameter for\n`bedtools genomecov` (step `bam_to_bedgraph`) only in a case when input `scale` is not provided. All logic is\nimplemented inside `bedtools-genomecov.cwl`.\n\n`bigwig_filename` defines the output name only for generated bigWig file. `bedgraph_filename` defines the output name\nfor generated bedGraph file and can influence on generated bigWig filename in case when `bigwig_filename` is not provided.\n\nAll workflow inputs and outputs don't have `format` field to avoid format incompatibility errors when workflow is used\nas subworkflow.",
            "id": "#bam-bedgraph-bigwig.cwl",
            "http://schema.org/name": "bam-bedgraph-bigwig",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file",
                    "label": "bedGraph output file",
                    "doc": "Output bedGraph file",
                    "id": "#bam-bedgraph-bigwig.cwl/bedgraph_file"
                },
                {
                    "type": "File",
                    "outputSource": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bigwig_file",
                    "doc": "Output bigWig file",
                    "id": "#bam-bedgraph-bigwig.cwl/bigwig_file"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { var ext = (inputs.depth == \"-bg\" || inputs.depth == \"-bga\")?\".bedGraph\":\".tab\"; return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.') + ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bedtools2:v2.26.0"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 17,
                        "prefix": "-g"
                    },
                    "doc": "Input genome file. Needed only when -i flag. The genome file is tab delimited <chromName><TAB><chromSize>\n",
                    "id": "#bedtools-genomecov.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#bedtools-genomecov.cwl/depth/depth",
                            "symbols": [
                                "#bedtools-genomecov.cwl/depth/depth/-bg",
                                "#bedtools-genomecov.cwl/depth/depth/-bga",
                                "#bedtools-genomecov.cwl/depth/depth/-d",
                                "#bedtools-genomecov.cwl/depth/depth/-dz"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Report the depth type. By default, bedtools genomecov will compute a histogram of coverage\nfor the genome file provided (intputs.chrom_length_file)\n",
                    "id": "#bedtools-genomecov.cwl/depth"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-du"
                    },
                    "doc": "Change strand af the mate read (so both reads from the same strand) useful for strand specific.\nWorks for BAM files only\n",
                    "id": "#bedtools-genomecov.cwl/du"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-fs"
                    },
                    "doc": "Set fixed fragment size\n",
                    "id": "#bedtools-genomecov.cwl/fragment_size"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16,
                        "valueFrom": "${\n  var prefix = ((/.*\\.bam$/i).test(inputs.input_file.path))?'-ibam':'-i';\n  return [prefix, inputs.input_file.path];\n}\n"
                    },
                    "doc": "The input file can be in BAM format (Note: BAM must be sorted by position) or <bed/gff/vcf>.\nPrefix is selected on the base of input file extension\n",
                    "id": "#bedtools-genomecov.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "-3"
                    },
                    "doc": "Calculate coverage of 3\" positions (instead of entire interval)\n",
                    "id": "#bedtools-genomecov.cwl/m3"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "-5"
                    },
                    "doc": "Calculate coverage of 5\" positions (instead of entire interval)\n",
                    "id": "#bedtools-genomecov.cwl/m5"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-scale",
                        "valueFrom": "${\n  if (inputs.scale){\n    return null;\n  } else if (inputs.mapped_reads_number) {\n    return 1000000/inputs.mapped_reads_number;\n  } else {\n    return null;\n  }\n}\n"
                    },
                    "doc": "Optional parameter to calculate scale as 1000000/mapped_reads_number if inputs.scale is not provided\n",
                    "id": "#bedtools-genomecov.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "-max"
                    },
                    "doc": "Combine all positions with a depth >= max into\na single bin in the histogram. Irrelevant\nfor -d and -bedGraph\n- (INTEGER)\n",
                    "id": "#bedtools-genomecov.cwl/max"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Name for generated output file\n",
                    "id": "#bedtools-genomecov.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-pc"
                    },
                    "doc": "pair-end chip seq experiment\n",
                    "id": "#bedtools-genomecov.cwl/pairchip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "-scale"
                    },
                    "doc": "Scale the coverage by a constant factor.\nEach coverage value is multiplied by this factor before being reported.\nUseful for normalizing coverage by, e.g., reads per million (RPM).\n- Default is 1.0; i.e., unscaled.\n- (FLOAT)\n",
                    "id": "#bedtools-genomecov.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "-split"
                    },
                    "doc": "treat \"split\" BAM or BED12 entries as distinct BED intervals.\nwhen computing coverage.\nFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\nto infer the blocks for computing coverage.\nFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\nfields (i.e., columns 10,11,12).\n",
                    "id": "#bedtools-genomecov.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "-strand"
                    },
                    "doc": "Calculate coverage of intervals from a specific strand.\nWith BED files, requires at least 6 columns (strand is column 6).\n- (STRING): can be + or -\n",
                    "id": "#bedtools-genomecov.cwl/strand"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "doc": "Generated genome coverage output file\n",
                    "id": "#bedtools-genomecov.cwl/genome_coverage_file",
                    "outputBinding": {
                        "glob": "${\n    return inputs.output_filename ? inputs.output_filename : default_output_filename();\n}\n"
                    }
                }
            ],
            "stdout": "${\n    return inputs.output_filename ? inputs.output_filename : default_output_filename();\n}\n",
            "baseCommand": [
                "bedtools",
                "genomecov"
            ],
            "doc": "Tool calculates genome coverage from input bam/bed/gff/vcf using `bedtools genomecov`\n\nDepending on `input_file` extension additional prefix is used: if `*.bam` use `-ibam`, else use `-i`.\n\n`scale` and `mapped_reads_number` inputs result in the same parameter `-scale`. If `scale` is not provided, check if\n`mapped_reads_number` is not null and calculate `-scale` as `1000000/mapped_reads_number`. If both inputs are\nnull, `bedtools genomecov` will use its default scaling value.\n\n`default_output_filename` function returns default output filename and is used when `output_filename` is not provided.\nDefault output file extention is `.tab`. If bedGraph should be generated (check flags `inputs.depth`), extension is\nupdated to `.bedGraph`. Default basename of the output file is generated on the base of `input_file` basename.\n",
            "id": "#bedtools-genomecov.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bedtools-metadata.yaml",
                "http://schema.org/name": "bedtools",
                "http://schema.org/about": "A software suite for the comparison, manipulation and annotation of genomic features in browser extensible data (BED) and general feature format (GFF) format. BEDTools also supports the comparison of sequence alignments in BAM format to both BED and GFF features. The tools are extremely efficient and allow the user to compare large datasets (e.g. next-generation sequencing data) with both public and custom genome annotation tracks. BEDTools can be combined with one another as well as with standard UNIX commands, thus facilitating routine genomics tasks as well as pipelines that can quickly answer intricate questions of large genomic datasets.\n",
                "http://schema.org/url": "http://bedtools.readthedocs.org",
                "http://schema.org/codeRepository": "https://github.com/arq5x/bedtools2",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-2.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "2.26.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bedtools-metadata.yaml/10.1093/bioinformatics/btq033",
                        "datePublished": "28 January 2010",
                        "http://schema.org/name": "BEDTools: a flexible suite of utilities for comparing genomic features",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron R. Quinlan"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ira M. Hall"
                            }
                        ],
                        "http://schema.org/url": "https://academic.oup.com/bioinformatics/article-lookup/doi/10.1093/bioinformatics/btq033"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/arq5x/bedtools2/issues",
                    "https://groups.google.com/forum/#!forum/bedtools-discuss"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "University of Utah",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron Quinlan",
                                "http://schema.org/description": "Main author"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Neil Kindlon",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bedtools-genomecov",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bedtools-genomecov.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: bedtools genomecov [OPTIONS] -i <bed/gff/vcf> -g <genome>\n\nOptions:\n\t-ibam\t\tThe input file is in BAM format.\n\t\t\tNote: BAM _must_ be sorted by position\n\n\t-d\t\tReport the depth at each genome position (with one-based coordinates).\n\t\t\tDefault behavior is to report a histogram.\n\n\t-dz\t\tReport the depth at each genome position (with zero-based coordinates).\n\t\t\tReports only non-zero positions.\n\t\t\tDefault behavior is to report a histogram.\n\n\t-bg\t\tReport depth in BedGraph format. For details, see:\n\t\t\tgenome.ucsc.edu/goldenPath/help/bedgraph.html\n\n\t-bga\t\tReport depth in BedGraph format, as above (-bg).\n\t\t\tHowever with this option, regions with zero\n\t\t\tcoverage are also reported. This allows one to\n\t\t\tquickly extract all regions of a genome with 0\n\t\t\tcoverage by applying: \"grep -w 0$\" to the output.\n\n\t-split\t\tTreat \"split\" BAM or BED12 entries as distinct BED intervals.\n\t\t\twhen computing coverage.\n\t\t\tFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\n\t\t\tto infer the blocks for computing coverage.\n\t\t\tFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\n\t\t\tfields (i.e., columns 10,11,12).\n\n\t-strand\t\tCalculate coverage of intervals from a specific strand.\n\t\t\tWith BED files, requires at least 6 columns (strand is column 6).\n\t\t\t- (STRING): can be + or -\n\n\t-pc\t\tCalculate coverage of pair-end fragments.\n\t\t\tWorks for BAM files only\n\t-fs\t\tForce to use provided fragment size instead of read length\n\t\t\tWorks for BAM files only\n\t-du\t\tChange strand af the mate read (so both reads from the same strand) useful for strand specific\n\t\t\tWorks for BAM files only\n\t-5\t\tCalculate coverage of 5\" positions (instead of entire interval).\n\n\t-3\t\tCalculate coverage of 3\" positions (instead of entire interval).\n\n\t-max\t\tCombine all positions with a depth >= max into\n\t\t\ta single bin in the histogram. Irrelevant\n\t\t\tfor -d and -bedGraph\n\t\t\t- (INTEGER)\n\n\t-scale\t\tScale the coverage by a constant factor.\n\t\t\tEach coverage value is multiplied by this factor before being reported.\n\t\t\tUseful for normalizing coverage by, e.g., reads per million (RPM).\n\t\t\t- Default is 1.0; i.e., unscaled.\n\t\t\t- (FLOAT)\n\n\t-trackline\tAdds a UCSC/Genome-Browser track line definition in the first line of the output.\n\t\t\t- See here for more details about track line definition:\n\t\t\t      http://genome.ucsc.edu/goldenPath/help/bedgraph.html\n\t\t\t- NOTE: When adding a trackline definition, the output BedGraph can be easily\n\t\t\t      uploaded to the Genome Browser as a custom track,\n\t\t\t      BUT CAN NOT be converted into a BigWig file (w/o removing the first line).\n\n\t-trackopts\tWrites additional track line definition parameters in the first line.\n\t\t\t- Example:\n\t\t\t   -trackopts 'name=\"My Track\" visibility=2 color=255,30,30'\n\t\t\t   Note the use of single-quotes if you have spaces in your parameters.\n\t\t\t- (TEXT)\n\nNotes:\n\t(1) The genome file should tab delimited and structured as follows:\n\t <chromName><TAB><chromSize>\n\n\tFor example, Human (hg19):\n\tchr1\t249250621\n\tchr2\t243199373\n\t...\n\tchr18_gl000207_random\t4262\n\n\t(2) The input BED (-i) file must be grouped by chromosome.\n\t A simple \"sort -k 1,1 <BED> > <BED>.sorted\" will suffice.\n\n\t(3) The input BAM (-ibam) file must be sorted by position.\n\t A \"samtools sort <BAM>\" should suffice.\n\nTips:\n\tOne can use the UCSC Genome Browser's MySQL database to extract\n\tchromosome sizes. For example, H. sapiens:\n\n\tmysql --user=genome --host=genome-mysql.cse.ucsc.edu -A -e \\\n\t\"select chrom, size from hg19.chromInfo\" > hg19.genome\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function(ext) { if (inputs.output_filename != \"\" && !ext){ return inputs.output_filename; } ext = ext || \".sam\"; var root = \"\"; if (inputs.output_filename != \"\"){ root = inputs.output_filename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.output_filename+ext:root+ext; } else if (Array.isArray(inputs.upstream_filelist) && inputs.upstream_filelist.length > 0){ root = inputs.upstream_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.upstream_filelist[0].basename+ext:root+ext; } else if (inputs.upstream_filelist != null){ root = inputs.upstream_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.upstream_filelist.basename+ext:root+ext; } else if (Array.isArray(inputs.downstream_filelist) && inputs.downstream_filelist.length > 0){ root = inputs.downstream_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.downstream_filelist[0].basename+ext:root+ext; } else if (inputs.downstream_filelist != null){ root = inputs.downstream_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.downstream_filelist.basename+ext:root+ext; } else if (Array.isArray(inputs.crossbow_filelist) && inputs.crossbow_filelist.length > 0){ root = inputs.crossbow_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.crossbow_filelist[0].basename+ext:root+ext; } else if (inputs.crossbow_filelist != null){ root = inputs.crossbow_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.crossbow_filelist.basename+ext:root+ext; } else { return null; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bowtie:v1.2.0"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-B"
                    },
                    "doc": "--offbase <int> leftmost ref offset = <int> in bowtie output (default: 0)\n",
                    "id": "#bowtie-alignreads.cwl/B"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-C"
                    },
                    "doc": "reads and index are in colorspace\n",
                    "id": "#bowtie-alignreads.cwl/C"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-I"
                    },
                    "doc": "--minins <int>  minimum insert size for paired-end alignment (default: 0)\n",
                    "id": "#bowtie-alignreads.cwl/I"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-M"
                    },
                    "doc": "<int> like -m, but reports 1 random hit (MAPQ=0)\n",
                    "id": "#bowtie-alignreads.cwl/M"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-Q"
                    },
                    "doc": "QV file(s) corresponding to CSFASTA inputs; use with -f -C\n",
                    "id": "#bowtie-alignreads.cwl/Q"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--Q1"
                    },
                    "doc": "--Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n",
                    "id": "#bowtie-alignreads.cwl/Q1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--Q2"
                    },
                    "doc": "--Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n",
                    "id": "#bowtie-alignreads.cwl/Q2"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-X"
                    },
                    "doc": "--maxins <int>  maximum insert size for paired-end alignment (default: 250)\n",
                    "id": "#bowtie-alignreads.cwl/X"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-a"
                    },
                    "doc": "--all           report all alignments per read (much slower than low -k)\n",
                    "id": "#bowtie-alignreads.cwl/a"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--al",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>       write aligned reads/pairs to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/aligned_prefix"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--best"
                    },
                    "doc": "hits guaranteed best stratum; ties broken by quality\n",
                    "id": "#bowtie-alignreads.cwl/best"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-c"
                    },
                    "doc": "query sequences given on cmd line (as <mates>, <singles>)\n",
                    "id": "#bowtie-alignreads.cwl/c"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chunkmbs"
                    },
                    "doc": "<int>   max megabytes of RAM for best-first search frames (def: 64)\n",
                    "id": "#bowtie-alignreads.cwl/chunkmbs"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-3"
                    },
                    "doc": "--trim3 <int>   trim <int> bases from 3' (right) end of reads\n",
                    "id": "#bowtie-alignreads.cwl/clip_3p_end"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-5"
                    },
                    "doc": "--trim5 <int>   trim <int> bases from 5' (left) end of reads\n",
                    "id": "#bowtie-alignreads.cwl/clip_5p_end"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-cqual"
                    },
                    "doc": "print original colorspace quals, not decoded quals\n",
                    "id": "#bowtie-alignreads.cwl/col_cqual"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-cseq"
                    },
                    "doc": "print aligned colorspace seqs as colors, not decoded bases\n",
                    "id": "#bowtie-alignreads.cwl/col_cseq"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-keepends"
                    },
                    "doc": "keep nucleotides at extreme ends of decoded alignment\n",
                    "id": "#bowtie-alignreads.cwl/col_keepends"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 86,
                        "prefix": "-12"
                    },
                    "doc": "Comma-separated list of files containing Crossbow-style reads.\nCan be a mixture of paired and unpaired.  Specify \"-\"for stdin.\n",
                    "id": "#bowtie-alignreads.cwl/crossbow_filelist"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 85
                    },
                    "doc": "Comma-separated list of files containing downstream mates (or the\nsequences themselves if -c is set) paired with mates in <m1>\n",
                    "id": "#bowtie-alignreads.cwl/downstream_filelist"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-e"
                    },
                    "doc": "--maqerr <int>  max sum of mismatch quals across alignment for -n (def: 70)\n",
                    "id": "#bowtie-alignreads.cwl/e"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "doc": "query input files are (multi-)FASTA .fa/.mfa\n",
                    "id": "#bowtie-alignreads.cwl/f"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ff"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/ff"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fr"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/fr"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fullref"
                    },
                    "doc": "write entire ref name (default: only up to 1st space)\n",
                    "id": "#bowtie-alignreads.cwl/fullref"
                },
                {
                    "type": "Directory",
                    "inputBinding": {
                        "position": 81,
                        "valueFrom": "${\n    for (var i = 0; i < self.listing.length; i++) {\n        if (self.listing[i].path.split('.').slice(-3).join('.') == 'rev.1.ebwt' ||\n            self.listing[i].path.split('.').slice(-3).join('.') == 'rev.1.ebwtl'){\n          return self.listing[i].path.split('.').slice(0,-3).join('.');\n        }\n    }\n    return null;\n}\n"
                    },
                    "doc": "Folder with Bowtie indices\n",
                    "id": "#bowtie-alignreads.cwl/indices_folder"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--integer-quals"
                    },
                    "doc": "qualities are given as space-separated integers (not ASCII)\n",
                    "id": "#bowtie-alignreads.cwl/integer_quals"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-k"
                    },
                    "doc": "<int>           report up to <int> good alignments per read (default: 1)\n",
                    "id": "#bowtie-alignreads.cwl/k"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-l"
                    },
                    "doc": "--seedlen <int> seed length for -n (default: 28)\n",
                    "id": "#bowtie-alignreads.cwl/l"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--large-index"
                    },
                    "doc": "force usage of a 'large' index, even if a small one is present\n",
                    "id": "#bowtie-alignreads.cwl/large_index"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-m"
                    },
                    "doc": "<int>           suppress all alignments if > <int> exist (def: no limit)\n",
                    "id": "#bowtie-alignreads.cwl/m"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--mapq"
                    },
                    "doc": "<int>       default mapping quality (MAPQ) to print for SAM alignments\n",
                    "id": "#bowtie-alignreads.cwl/mapq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--maxbts"
                    },
                    "doc": "<int>     max # backtracks for -n 2/3 (default: 125, 800 for --best)\n",
                    "id": "#bowtie-alignreads.cwl/maxbts"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--mm"
                    },
                    "doc": "use memory-mapped I/O for index; many 'bowtie's can share\n",
                    "id": "#bowtie-alignreads.cwl/mm"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--max",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>      write reads/pairs over -m limit to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/multimapped_prefix"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-n"
                    },
                    "doc": "--seedmms <int> max mismatches in seed (can be 0-3, default: -n 2)\n",
                    "id": "#bowtie-alignreads.cwl/n"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--nofw"
                    },
                    "doc": "--norc      do not align to forward/reverse-complement reference strand\n",
                    "id": "#bowtie-alignreads.cwl/nofw"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--nomaqround"
                    },
                    "doc": "disable Maq-like quality rounding for -n (nearest 10 <= 30)\n",
                    "id": "#bowtie-alignreads.cwl/nomaqround"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-o"
                    },
                    "doc": "--offrate <int> override offrate of index; must be >= index's offrate\n",
                    "id": "#bowtie-alignreads.cwl/o"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 90,
                        "valueFrom": "$(default_output_filename())"
                    },
                    "default": "",
                    "doc": "Generates default output filename on the base of upstream_filelist/downstream_filelist files\n",
                    "id": "#bowtie-alignreads.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--pairtries"
                    },
                    "doc": "<int>  max # attempts to find mate for anchor hit (default: 100)\n",
                    "id": "#bowtie-alignreads.cwl/pairtries"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--phred33-quals"
                    },
                    "doc": "input quals are Phred+33 (default)\n",
                    "id": "#bowtie-alignreads.cwl/phred33_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--phred64-quals"
                    },
                    "doc": "input quals are Phred+64 (same as --solexa1.3-quals)\n",
                    "id": "#bowtie-alignreads.cwl/phred64_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-q"
                    },
                    "doc": "query input files are FASTQ .fq/.fastq (default)\n",
                    "id": "#bowtie-alignreads.cwl/q"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quiet"
                    },
                    "doc": "print nothing but the alignments\n",
                    "id": "#bowtie-alignreads.cwl/quiet"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-r"
                    },
                    "doc": "query input files are raw one-sequence-per-line\n",
                    "id": "#bowtie-alignreads.cwl/r"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--reads-per-batch"
                    },
                    "doc": "# of reads to read from input file at once (default: 16)\n",
                    "id": "#bowtie-alignreads.cwl/reads_per_batch"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--refidx"
                    },
                    "doc": "refer to ref. seqs by 0-based index rather than name\n",
                    "id": "#bowtie-alignreads.cwl/refidx"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--refout"
                    },
                    "doc": "write alignments to files refXXXXX.map, 1 map per reference\n",
                    "id": "#bowtie-alignreads.cwl/refout"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--rf"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/rf"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-s"
                    },
                    "doc": "--skip <int>    skip the first <int> reads/pairs in the input\n",
                    "id": "#bowtie-alignreads.cwl/s"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-S"
                    },
                    "doc": "--sam write hits in SAM format\n",
                    "id": "#bowtie-alignreads.cwl/sam"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-RG"
                    },
                    "doc": "<text>    add <text> (usually \"lab=value\") to @RG line of SAM header\n",
                    "id": "#bowtie-alignreads.cwl/sam_RG"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-nohead"
                    },
                    "doc": "supppress header lines (starting with @) for SAM output\n",
                    "id": "#bowtie-alignreads.cwl/sam_nohead"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-nosq"
                    },
                    "doc": "supppress @SQ header lines for SAM output\n",
                    "id": "#bowtie-alignreads.cwl/sam_nosq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seed"
                    },
                    "doc": "<int>       seed for random number generator\n",
                    "id": "#bowtie-alignreads.cwl/seed"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--shmem"
                    },
                    "doc": "use shared mem for index\n",
                    "id": "#bowtie-alignreads.cwl/shmem"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--snpfrac"
                    },
                    "doc": "<dec>    approx. fraction of SNP bases (e.g. 0.001); sets --snpphred\n",
                    "id": "#bowtie-alignreads.cwl/snpfrac"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--snpphred"
                    },
                    "doc": "<int>   Phred penalty for SNP when decoding colorspace (def: 30)\n",
                    "id": "#bowtie-alignreads.cwl/snpphred"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--solexa1.3-quals"
                    },
                    "doc": "input quals are from GA Pipeline ver. >= 1.3\n",
                    "id": "#bowtie-alignreads.cwl/solexa1.3_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--solexa-quals"
                    },
                    "doc": "input quals are from GA Pipeline ver. < 1.3\n",
                    "id": "#bowtie-alignreads.cwl/solexa_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--strata"
                    },
                    "doc": "hits in sub-optimal strata aren't reported (requires --best)\n",
                    "id": "#bowtie-alignreads.cwl/strata"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--suppress"
                    },
                    "doc": "<cols>  suppresses given columns (comma-delim'ed) in default output\n",
                    "id": "#bowtie-alignreads.cwl/suppress"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-t"
                    },
                    "doc": "--time          print wall-clock time taken by search phases\n",
                    "id": "#bowtie-alignreads.cwl/t"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-p"
                    },
                    "doc": "--threads <int> number of alignment threads to launch (default: 1)\n",
                    "id": "#bowtie-alignreads.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-u"
                    },
                    "doc": "--qupto <int>   stop after first <int> reads/pairs (excl. skipped reads)\n",
                    "id": "#bowtie-alignreads.cwl/u"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--un",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>       write unaligned reads/pairs to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/unaligned_prefix"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 83
                    },
                    "doc": "Comma-separated list of files containing upstream mates (or the\nsequences themselves, if -c is set) paired with mates in <m2>\n",
                    "id": "#bowtie-alignreads.cwl/upstream_filelist"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-v"
                    },
                    "doc": "report end-to-end hits w/ <=v mismatches; ignore qualities\n",
                    "id": "#bowtie-alignreads.cwl/v"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--verbose"
                    },
                    "doc": "verbose output (for debugging)\n",
                    "id": "#bowtie-alignreads.cwl/verbose"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-y"
                    },
                    "doc": "--tryhard try hard to find valid alignments, at the expense of speed\n",
                    "id": "#bowtie-alignreads.cwl/y"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.aligned_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/aligned_fastq"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(default_output_filename(\".bw\"))"
                    },
                    "id": "#bowtie-alignreads.cwl/log_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var mappedRegex = /alignment\\:.*/;\n  return parseInt(self[0].contents.match(mappedRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.multimapped_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/multimapped_fastq"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "*ref*.map*"
                    },
                    "id": "#bowtie-alignreads.cwl/refout_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(default_output_filename())"
                    },
                    "id": "#bowtie-alignreads.cwl/sam_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var totalRegex = /processed\\:.*/;\n  return parseInt(self[0].contents.match(totalRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/total_reads_number"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.unaligned_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/unaligned_fastq"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var unmappedRegex = /align\\:.*/;\n  return parseInt(self[0].contents.match(unmappedRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/unmapped_reads_number"
                }
            ],
            "baseCommand": [
                "bowtie"
            ],
            "arguments": [
                {
                    "valueFrom": "${\n  if (inputs.upstream_filelist && inputs.downstream_filelist){\n    return \"-1\";\n  }\n  return null;\n}\n",
                    "position": 82
                },
                {
                    "valueFrom": "${\n  if (inputs.upstream_filelist && inputs.downstream_filelist){\n    return \"-2\";\n  }\n  return null;\n}\n",
                    "position": 84
                },
                {
                    "valueFrom": "${\n  return ' 2> ' + default_output_filename(\".bw\");\n}\n",
                    "position": 100000,
                    "shellQuote": false
                }
            ],
            "doc": "Tool maps input raw reads files to reference genome using Bowtie.\n\n`default_output_filename` function returns default name for SAM output and log files. In case when `sam` and\n`output_filename` inputs are not set, default filename will have `.sam` extension but format may not correspond SAM\nspecification. To set output filename manually use `output_filename` input. Default output filename is based on\n`output_filename` or basename of `upstream_filelist`, `downstream_filelist` or `crossbow_filelist` file (if array,\nthe first file in array is taken). If function is called without argenments and `output_filename` input is set, it\nwill be returned from the function.\n\nFor single-end input data any of the `upstream_filelist` or `downstream_filelist` inputs can be used.\n\nLog filename (`log_file` output) is generated by `default_output_filename` function with ex='.bw'\n\n`indices_folder` defines folder to contain Bowtie indices. Based on the first found file with `rev.1.ebwt` or\n`rev.1.ebwtl` extension, bowtie index prefix is returned from input's `valueFrom` field.\n",
            "id": "#bowtie-alignreads.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bowtie-metadata.yaml",
                "http://schema.org/name": "bowtie2",
                "http://schema.org/about": "Bowtie is an ultrafast, memory-efficient short read aligner. It aligns short DNA sequences (reads)\nto the human genome at a rate of over 25 million 35-bp reads per hour. Bowtie indexes the genome\nwith a Burrows-Wheeler index to keep its memory footprint small: typically about 2.2 GB for\nthe human genome (2.9 GB for paired-end).\n",
                "http://schema.org/url": "http://bowtie-bio.sourceforge.net/index.shtml",
                "http://schema.org/codeRepository": "https://github.com/BenLangmead/bowtie",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "1.2.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bowtie-metadata.yaml/10.1186/gb-2009-10-3-r25",
                        "datePublished": "04 March 2009",
                        "http://schema.org/name": "Ultrafast and memory-efficient alignment of short DNA sequences to the human genome",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ben Langmead"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Steven L Salzberg"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Cole Trapnell"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Mihai Pop"
                            }
                        ],
                        "http://schema.org/url": "https://genomebiology.biomedcentral.com/articles/10.1186/gb-2009-10-3-r25"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/BenLangmead/bowtie/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Johns Hopkins University",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ben Langmead",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bowtie",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bowtie.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\nbowtie [options]* <ebwt> {-1 <m1> -2 <m2> | --12 <r> | <s>} [<hit>]\n\n  <m1>    Comma-separated list of files containing upstream mates (or the\n          sequences themselves, if -c is set) paired with mates in <m2>\n  <m2>    Comma-separated list of files containing downstream mates (or the\n          sequences themselves if -c is set) paired with mates in <m1>\n  <r>     Comma-separated list of files containing Crossbow-style reads.  Can be\n          a mixture of paired and unpaired.  Specify \"-\" for stdin.\n  <s>     Comma-separated list of files containing unpaired reads, or the\n          sequences themselves, if -c is set.  Specify \"-\" for stdin.\n  <hit>   File to write hits to (default: stdout)\nInput:\n  -q                 query input files are FASTQ .fq/.fastq (default)\n  -f                 query input files are (multi-)FASTA .fa/.mfa\n  -r                 query input files are raw one-sequence-per-line\n  -c                 query sequences given on cmd line (as <mates>, <singles>)\n  -C                 reads and index are in colorspace\n  -Q/--quals <file>  QV file(s) corresponding to CSFASTA inputs; use with -f -C\n  --Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n  -s/--skip <int>    skip the first <int> reads/pairs in the input\n  -u/--qupto <int>   stop after first <int> reads/pairs (excl. skipped reads)\n  -5/--trim5 <int>   trim <int> bases from 5' (left) end of reads\n  -3/--trim3 <int>   trim <int> bases from 3' (right) end of reads\n  --phred33-quals    input quals are Phred+33 (default)\n  --phred64-quals    input quals are Phred+64 (same as --solexa1.3-quals)\n  --solexa-quals     input quals are from GA Pipeline ver. < 1.3\n  --solexa1.3-quals  input quals are from GA Pipeline ver. >= 1.3\n  --integer-quals    qualities are given as space-separated integers (not ASCII)\n  --large-index      force usage of a 'large' index, even if a small one is present\nAlignment:\n  -v <int>           report end-to-end hits w/ <=v mismatches; ignore qualities\n    or\n  -n/--seedmms <int> max mismatches in seed (can be 0-3, default: -n 2)\n  -e/--maqerr <int>  max sum of mismatch quals across alignment for -n (def: 70)\n  -l/--seedlen <int> seed length for -n (default: 28)\n  --nomaqround       disable Maq-like quality rounding for -n (nearest 10 <= 30)\n  -I/--minins <int>  minimum insert size for paired-end alignment (default: 0)\n  -X/--maxins <int>  maximum insert size for paired-end alignment (default: 250)\n  --fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n  --nofw/--norc      do not align to forward/reverse-complement reference strand\n  --maxbts <int>     max # backtracks for -n 2/3 (default: 125, 800 for --best)\n  --pairtries <int>  max # attempts to find mate for anchor hit (default: 100)\n  -y/--tryhard       try hard to find valid alignments, at the expense of speed\n  --chunkmbs <int>   max megabytes of RAM for best-first search frames (def: 64)\n --reads-per-batch   # of reads to read from input file at once (default: 16)\nReporting:\n  -k <int>           report up to <int> good alignments per read (default: 1)\n  -a/--all           report all alignments per read (much slower than low -k)\n  -m <int>           suppress all alignments if > <int> exist (def: no limit)\n  -M <int>           like -m, but reports 1 random hit (MAPQ=0); requires --best\n  --best             hits guaranteed best stratum; ties broken by quality\n  --strata           hits in sub-optimal strata aren't reported (requires --best)\nOutput:\n  -t/--time          print wall-clock time taken by search phases\n  -B/--offbase <int> leftmost ref offset = <int> in bowtie output (default: 0)\n  --quiet            print nothing but the alignments\n  --refout           write alignments to files refXXXXX.map, 1 map per reference\n  --refidx           refer to ref. seqs by 0-based index rather than name\n  --al <fname>       write aligned reads/pairs to file(s) <fname>\n  --un <fname>       write unaligned reads/pairs to file(s) <fname>\n  --max <fname>      write reads/pairs over -m limit to file(s) <fname>\n  --suppress <cols>  suppresses given columns (comma-delim'ed) in default output\n  --fullref          write entire ref name (default: only up to 1st space)\nColorspace:\n  --snpphred <int>   Phred penalty for SNP when decoding colorspace (def: 30)\n     or\n  --snpfrac <dec>    approx. fraction of SNP bases (e.g. 0.001); sets --snpphred\n  --col-cseq         print aligned colorspace seqs as colors, not decoded bases\n  --col-cqual        print original colorspace quals, not decoded quals\n  --col-keepends     keep nucleotides at extreme ends of decoded alignment\nSAM:\n  -S/--sam           write hits in SAM format\n  --mapq <int>       default mapping quality (MAPQ) to print for SAM alignments\n  --sam-nohead       supppress header lines (starting with @) for SAM output\n  --sam-nosq         supppress @SQ header lines for SAM output\n  --sam-RG <text>    add <text> (usually \"lab=value\") to @RG line of SAM header\nPerformance:\n  -o/--offrate <int> override offrate of index; must be >= index's offrate\n  -p/--threads <int> number of alignment threads to launch (default: 1)\n  --mm               use memory-mapped I/O for index; many 'bowtie's can share\n  --shmem            use shared mem for index; many 'bowtie's can share\nOther:\n  --seed <int>       seed for random number generator\n  --verbose          verbose output (for debugging)\n  --version          print version information and quit\n  -h/--help          print this usage message\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_output_prefix = function() { if (inputs.output_prefix) { return inputs.output_prefix; } var root = inputs.star_alignment_report.basename.split('.').slice(0,-1).join('.'); var suffix = \"_collected_statistics_report\"; return (root == \"\")?inputs.star_alignment_report.basename+suffix:root+suffix; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "rackspacedot/python37"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--bamstats"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/bam_statistics_report"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "--bowtie"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/bowtie_alignment_report"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--isoforms"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/isoforms_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--output",
                        "valueFrom": "$(get_output_prefix())"
                    },
                    "default": "",
                    "id": "#collect-statistics-rna-quantseq.cwl/output_prefix"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--paired"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/paired_end"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/usr/bin/env python\nimport os\nimport sys\nimport argparse\nimport yaml\nimport math\nimport re\n\n\ndef cut_int(s):\n    return int(s.strip().split()[0])\n\ndef cut_float(s):\n    return float(s.strip().split()[0])\n\ndef cut_percent(s):\n    return float(s.strip().replace(\"%\", \"\").split()[0])\n\n\nADAPTER_TRIMMING = {\n    \"Trimming mode\": {\n        \"alias\": \"trimming mode\",\n        \"function\": str,\n        \"pair_end_specific\": False\n    },\n    \"Quality Phred score cutoff\": {\n        \"alias\": \"quality phred score cutoff\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"Quality encoding type selected\": {\n        \"alias\": \"quality encoding type\",\n        \"function\": str,\n        \"pair_end_specific\": False\n    },\n    \"Adapter sequence\": {\n        \"alias\": \"adapter sequence\",\n        \"function\": str,\n        \"pair_end_specific\": False\n    },\n    \"Minimum required adapter overlap\": {\n        \"alias\": \"minimum required adapter overlap\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"Maximum trimming error rate\": {\n        \"alias\": \"maximum trimming error rate\",\n        \"function\": cut_float,\n        \"pair_end_specific\": False\n    },\n    \"Minimum required sequence length\": {\n        \"alias\": \"minimum required read length\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"Total number of sequences analysed\": {\n        \"alias\": \"number of reads/pairs analysed for length validation\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"Number of sequence pairs removed\": {\n        \"alias\": \"reads/pairs removed because of length cutoff\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"Sequences removed because they became shorter\": {\n        \"alias\": \"reads/pairs removed because of length cutoff\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"order\": [\"fastq\",\n            \"trimming mode\",\n            \"adapter sequence\",\n            \"number of reads/pairs analysed for length validation\",\n            \"reads/pairs removed because of length cutoff\",\n            \"minimum required read length\",\n            \"quality phred score cutoff\",\n            \"quality encoding type\",\n            \"minimum required adapter overlap\",\n            \"maximum trimming error rate\"]\n}\n\n\nALIGNMENT = {\n    \"Number of input reads\": {\n        \"alias\": \"total reads/pairs processed\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"Uniquely mapped reads number\": {\n        \"alias\": \"uniquely mapped reads/pairs number\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"Mismatch rate per base, %\": {\n        \"alias\": \"mismatch rate per base, %\",\n        \"function\": cut_percent,\n        \"pair_end_specific\": False\n    },\n    \"Deletion rate per base\": {\n        \"alias\": \"deletion rate per base, %\",\n        \"function\": cut_percent,\n        \"pair_end_specific\": False\n    },\n    \"Number of reads mapped to multiple loci\": {\n        \"alias\": \"reads/pairs mapped to multiple loci\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"Number of reads mapped to too many loci\": {\n        \"alias\": \"reads/pairs suppressed due to mapping to too many loci\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"reads unmapped: too many mismatches\": {\n        \"alias\": \"reads/pairs unmapped due to too many mismatches, %\",\n        \"function\": cut_percent,\n        \"pair_end_specific\": False\n    },\n    \"reads unmapped: too short\": {\n        \"alias\": \"reads/pairs unmapped due to too short, %\",\n        \"function\": cut_percent,\n        \"pair_end_specific\": False\n    },\n    \"reads unmapped: other\": {\n        \"alias\": \"reads/pairs unmapped due to other reasons, %\",\n        \"function\": cut_percent,\n        \"pair_end_specific\": False\n    },\n    \"reads with at least one reported alignment\": {\n        \"alias\": \"number of reads/pairs in ribosomal dna\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"order\": [\"total reads/pairs processed\",\n            \"uniquely mapped reads/pairs number\",\n            \"number of reads/pairs in transcriptome\",\n            \"number of reads/pairs in ribosomal dna\",\n            \"reads/pairs mapped to multiple loci\",\n            \"reads/pairs suppressed due to mapping to too many loci\",\n            \"reads/pairs unmapped due to too many mismatches, %\",\n            \"reads/pairs unmapped due to too short, %\",\n            \"reads/pairs unmapped due to other reasons, %\",\n            \"mismatch rate per base, %\",\n            \"deletion rate per base, %\"]\n}\n\n\nBAMSTATS = {\n    \"raw total sequences\": {\n        \"alias\": \"total reads/pairs\",\n        \"function\": int,\n        \"pair_end_specific\": True\n    },\n    \"reads mapped\": {\n        \"alias\": \"reads/pairs mapped\",\n        \"function\": int,\n        \"pair_end_specific\": True\n    },\n    \"reads unmapped\": {\n        \"alias\": \"reads/pairs unmapped\",\n        \"function\": int,\n        \"pair_end_specific\": True\n    },\n    \"average length\": {\n        \"alias\": \"reads average length\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"maximum length\": {\n        \"alias\": \"reads maximum length\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"average quality\": {\n        \"alias\": \"reads average quality\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"insert size average\": {\n        \"alias\": \"insert size average\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"insert size standard deviation\": {\n        \"alias\": \"insert size standard deviation\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"order\": [\"total reads/pairs\",\n            \"reads/pairs mapped\",\n            \"reads/pairs unmapped\",\n            \"reads average length\",\n            \"reads maximum length\",\n            \"reads average quality\",\n            \"insert size average\",\n            \"insert size standard deviation\"]\n}\n\n\ndef arg_parser():\n    general_parser = argparse.ArgumentParser()\n    general_parser.add_argument(\"--trim1\",           help=\"Path to Trimgalore report file for FASTQ 1\",           required=False)\n    general_parser.add_argument(\"--trim2\",           help=\"Path to Trimgalore report file for FASTQ 2\",           required=False)\n    general_parser.add_argument(\"--star\",            help=\"Path to STAR report file\",                             required=False)\n    general_parser.add_argument(\"--bowtie\",          help=\"Path to Bowtie report file\",                           required=False)\n    general_parser.add_argument(\"--bamstats\",        help=\"Path to bam statistics report file\",                   required=False)\n    general_parser.add_argument(\"--isoforms\",        help=\"Path to isoforms file\",                                required=False)\n    general_parser.add_argument(\"--paired\",          help=\"Process as paired-end\",                           action=\"store_true\")\n    general_parser.add_argument(\"--output\",          help=\"Output filename prefix\",               default=\"collected_statistics\")\n    return general_parser\n\n\ndef normalize_args(args, skip_list=[]):\n    normalized_args = {}\n    for key,value in args.__dict__.items():\n        if key not in skip_list:\n            normalized_args[key] = value if not value or os.path.isabs(value) else os.path.normpath(os.path.join(os.getcwd(), value))\n        else:\n            normalized_args[key]=value\n    return argparse.Namespace (**normalized_args)\n\n\ndef open_file(filename):\n    lines = []\n    with open(filename, 'r') as infile:\n        for line in infile:\n            if line.strip():\n                lines.append(line.strip())\n    return lines\n\n\ndef split_line(l, delimiter):\n    return [i.strip() for i in l.split(delimiter)]\n\n\ndef get_correspondent_key(data, long_k):\n    for short_k, v in data.items():\n        if short_k in long_k:\n            return v[\"alias\"], v[\"function\"], v[\"pair_end_specific\"]\n    raise Exception\n\n\ndef process_trimgalore_report(filepath, collected_results, header, key_dict, pair_end, delimiter):\n    if not collected_results.get(header, None):\n        collected_results[header] = {\"fastq\": []}\n    fastq = {}\n    for line in open_file(filepath):\n        try:\n            key, value = split_line(line, delimiter)\n            if \"Total reads processed\" in line:\n                fastq[\"total reads processed\"] = int(value.strip().replace(\",\",\"\"))\n            elif \"Reads with adapters\" in line:\n                fastq[\"reads with adapters\"] = int(value.strip().replace(\",\",\"\").split()[0])\n            else:\n                res_key, res_function, pair_end_specific = get_correspondent_key(key_dict, key)\n                if not collected_results[header].get(res_key, None):\n                    collected_results[header][res_key] = res_function(value)\n        except Exception:\n            pass\n    if not pair_end:\n        collected_results[header][\"number of reads/pairs analysed for length validation\"] = fastq[\"total reads processed\"]\n    collected_results[header][\"fastq\"].append(fastq)\n    if key_dict.get(\"order\", None):\n        collected_results[header] = {k: collected_results[header][k] for k in key_dict[\"order\"] if k in collected_results[header]}\n\n\ndef process_custom_report(filepath, collected_results, header, key_dict, pair_end, delimiter):\n    if not collected_results.get(header, None):\n        collected_results[header] = {}\n    for line in open_file(filepath):\n        try:\n            key, value = split_line(line, delimiter)\n            res_key, res_function, pair_end_specific = get_correspondent_key(key_dict, key)\n            if not collected_results[header].get(res_key, None):\n                if pair_end_specific and pair_end:\n                    collected_results[header][res_key] = res_function(res_function(value)/2)\n                else:\n                    collected_results[header][res_key] = res_function(value)\n        except Exception:\n            pass\n    if key_dict.get(\"order\", None):\n        collected_results[header] = {k: collected_results[header][k] for k in key_dict[\"order\"] if k in collected_results[header]}\n\n\ndef process_isoforms_report(filepath, collected_results, header, key_dict, pair_end):\n    if not collected_results.get(header, None):\n        collected_results[header] = {}\n    total_reads_index = None\n    used_reads = 0\n    for line in open_file(filepath):\n        if re.match ('.*RefseqId.*|.*GeneId.*|.*Chrom.*|.*TotalReads.*', line) and total_reads_index is None:\n            total_reads_index = line.split(',').index('TotalReads')\n            continue\n        line_splitted = line.split(',')\n        used_reads += int(line_splitted[total_reads_index])\n    if pair_end:\n        used_reads = int(used_reads/2)\n    collected_results[header][\"number of reads/pairs in transcriptome\"] = used_reads\n    if key_dict.get(\"order\", None):\n        collected_results[header] = {k: collected_results[header][k] for k in key_dict[\"order\"] if k in collected_results[header]}\n\n\ndef collect_stats(args):\n    collected_results = {}\n\n    if args.trim1:\n        process_trimgalore_report(filepath=args.trim1,\n                                collected_results=collected_results,\n                                header=\"adapter trimming statistics\",\n                                key_dict=ADAPTER_TRIMMING,\n                                pair_end=args.paired,\n                                delimiter=\":\")\n\n    if args.trim2:\n        process_trimgalore_report(filepath=args.trim2,\n                                collected_results=collected_results,\n                                header=\"adapter trimming statistics\",\n                                key_dict=ADAPTER_TRIMMING,\n                                pair_end=args.paired,\n                                delimiter=\":\")\n\n    if args.star:\n        process_custom_report(filepath=args.star,\n                            collected_results=collected_results,\n                            header=\"alignment statistics\",\n                            key_dict=ALIGNMENT,\n                            pair_end=args.paired,\n                            delimiter=\"|\")\n\n    if args.isoforms:\n        process_isoforms_report(filepath=args.isoforms,\n                                collected_results=collected_results,\n                                header=\"alignment statistics\",\n                                key_dict=ALIGNMENT,\n                                pair_end=args.paired)\n\n    if args.bowtie:\n        process_custom_report(filepath=args.bowtie,\n                            collected_results=collected_results,\n                            header=\"alignment statistics\",\n                            key_dict=ALIGNMENT,\n                            pair_end=args.paired,\n                            delimiter=\":\")\n\n    if args.bamstats:\n        process_custom_report(filepath=args.bamstats,\n                            collected_results=collected_results,\n                            header=\"BAM statistics\",\n                            key_dict=BAMSTATS,\n                            pair_end=args.paired,\n                            delimiter=\":\")\n\n    return (collected_results)\n\n\ndef export_results_yaml(collected_data, filepath):\n    with open(filepath+\".yaml\", 'w') as output_stream:\n        output_stream.write(yaml.dump(collected_data, width=1000, sort_keys=False))\n\n\ndef export_results_markdown(collected_data, filepath):\n    with open(filepath+\".md\", 'w') as output_stream:\n        for line in yaml.dump(collected_data, width=1000, sort_keys=False).split(\"\\n\"):\n            if not line.strip():\n                continue\n            if line.startswith(\"  - \"):\n                output_stream.write(line+\"\\n\")\n            elif line.startswith(\"    \"):\n                output_stream.write(\"<br>\"+line+\"\\n\")\n            elif line.startswith(\"  \"):\n                output_stream.write(\"- \"+line+\"\\n\")\n            else:\n                output_stream.write(\"### \"+line+\"\\n\")\n\n\ndef export_results_table(collected_data, filepath):\n    with open(filepath+\".tsv\", 'w') as output_stream:\n        total_reads= collected_data[\"alignment statistics\"][\"total reads/pairs processed\"]\n        uniquely_mapped_reads = collected_data[\"alignment statistics\"][\"uniquely mapped reads/pairs number\"]\n        uniquely_mapped_reads_dedup = collected_data[\"BAM statistics\"][\"reads/pairs mapped\"]\n        transcriptome_reads = collected_data[\"alignment statistics\"][\"number of reads/pairs in transcriptome\"]\n        multimapped_reads = collected_data[\"alignment statistics\"][\"reads/pairs suppressed due to mapping to too many loci\"]\n        not_transcriptome_reads = uniquely_mapped_reads_dedup - transcriptome_reads\n        unmapped_reads = total_reads - uniquely_mapped_reads - multimapped_reads\n        ribosomal_reads = collected_data[\"alignment statistics\"][\"number of reads/pairs in ribosomal dna\"]\n        reads_dedup = uniquely_mapped_reads - uniquely_mapped_reads_dedup\n\n        header = [\n                    \"Tags total\",\n                    \"Transcriptome\",\n                    \"Multi-mapped\",\n                    \"Outside annotation\",\n                    \"Unmapped\",\n                    \"Deduplicated\",\n                    \"Ribosomal contamination\",\n\n                    \"alignment statistics\",\n                    \"total reads/pairs processed\",\n                    \"uniquely mapped reads/pairs number\",\n                    \"reads/pairs mapped to multiple loci\",\n                    \"reads/pairs suppressed due to mapping to too many loci\",\n                    \"mismatch rate per base, %\",\n                    \"deletion rate per base, %\",\n                    \"reads/pairs unmapped due to too many mismatches, %\",\n                    \"reads/pairs unmapped due to too short, %\",\n                    \"reads/pairs unmapped due to other reasons, %\",\n                    \"number of reads/pairs in transcriptome\",\n                    \"number of reads/pairs in ribosomal dna\",\n\n                    \"BAM statistics\",\n                    \"total reads/pairs\",\n                    \"reads/pairs mapped\",\n                    \"reads/pairs unmapped\",\n                    \"insert size average\",\n                    \"insert size standard deviation\",\n                    \"reads average length\",\n                    \"reads average quality\",\n                    \"reads maximum length\"]\n\n        if collected_data.get(\"adapter trimming statistics\", None):\n            header.extend([\"adapter trimming statistics\",\n                        \"trimming mode\",\n                        \"adapter sequence\",\n                        \"quality phred score cutoff\",\n                        \"minimum required adapter overlap\",\n                        \"maximum trimming error rate\",\n                        \"minimum required read length\",\n                        \"number of reads/pairs analysed for length validation\",\n                        \"reads/pairs removed because of length cutoff\",\n                        \"fastq1: total reads processed\",\n                        \"fastq1: reads with adapters\"])\n            if len(collected_data[\"adapter trimming statistics\"][\"fastq\"]) > 1:\n                header.extend([\"fastq2: total reads processed\", \"fastq2: reads with adapters\"])\n\n        output_stream.write(\"\\t\".join(header)+\"\\n\")\n\n        data = [\n                total_reads,\n                transcriptome_reads,\n                multimapped_reads,\n                not_transcriptome_reads,\n                unmapped_reads,\n                reads_dedup,\n                ribosomal_reads,\n\n                \"\",\n                collected_data[\"alignment statistics\"][\"total reads/pairs processed\"],\n                collected_data[\"alignment statistics\"][\"uniquely mapped reads/pairs number\"],\n                collected_data[\"alignment statistics\"][\"reads/pairs mapped to multiple loci\"],\n                collected_data[\"alignment statistics\"][\"reads/pairs suppressed due to mapping to too many loci\"],\n                collected_data[\"alignment statistics\"][\"mismatch rate per base, %\"],\n                collected_data[\"alignment statistics\"][\"deletion rate per base, %\"],\n                collected_data[\"alignment statistics\"][\"reads/pairs unmapped due to too many mismatches, %\"],\n                collected_data[\"alignment statistics\"][\"reads/pairs unmapped due to too short, %\"],\n                collected_data[\"alignment statistics\"][\"reads/pairs unmapped due to other reasons, %\"],\n                collected_data[\"alignment statistics\"][\"number of reads/pairs in transcriptome\"],\n                collected_data[\"alignment statistics\"][\"number of reads/pairs in ribosomal dna\"],\n\n                \"\",\n                collected_data[\"BAM statistics\"][\"total reads/pairs\"],\n                collected_data[\"BAM statistics\"][\"reads/pairs mapped\"],\n                collected_data[\"BAM statistics\"][\"reads/pairs unmapped\"],\n                collected_data[\"BAM statistics\"][\"insert size average\"],\n                collected_data[\"BAM statistics\"][\"insert size standard deviation\"],\n                collected_data[\"BAM statistics\"][\"reads average length\"],\n                collected_data[\"BAM statistics\"][\"reads average quality\"],\n                collected_data[\"BAM statistics\"][\"reads maximum length\"]]\n\n        if collected_data.get(\"adapter trimming statistics\", None):\n            data.extend([\"\",\n                        collected_data[\"adapter trimming statistics\"][\"trimming mode\"],\n                        collected_data[\"adapter trimming statistics\"][\"adapter sequence\"],\n                        collected_data[\"adapter trimming statistics\"][\"quality phred score cutoff\"],\n                        collected_data[\"adapter trimming statistics\"][\"minimum required adapter overlap\"],\n                        collected_data[\"adapter trimming statistics\"][\"maximum trimming error rate\"],\n                        collected_data[\"adapter trimming statistics\"][\"minimum required read length\"],\n                        collected_data[\"adapter trimming statistics\"][\"number of reads/pairs analysed for length validation\"],\n                        collected_data[\"adapter trimming statistics\"][\"reads/pairs removed because of length cutoff\"],\n                        collected_data[\"adapter trimming statistics\"][\"fastq\"][0][\"total reads processed\"],\n                        collected_data[\"adapter trimming statistics\"][\"fastq\"][0][\"reads with adapters\"]])\n            if len(collected_data[\"adapter trimming statistics\"][\"fastq\"]) > 1:\n                data.extend([collected_data[\"adapter trimming statistics\"][\"fastq\"][1][\"total reads processed\"],\n                            collected_data[\"adapter trimming statistics\"][\"fastq\"][1][\"reads with adapters\"]])\n\n        output_stream.write(\"\\t\".join(str(l) for l in data))\n\n\ndef main(argsl=None):\n    if argsl is None:\n        argsl = sys.argv[1:]\n    args,_ = arg_parser().parse_known_args(argsl)\n    args = normalize_args(args, skip_list=[\"paired\"])\n    collected_data = collect_stats(args)\n    export_results_yaml(collected_data, args.output)\n    export_results_table(collected_data, args.output)\n    export_results_markdown(collected_data, args.output)\n\n\nif __name__ == \"__main__\":\n    sys.exit(main(sys.argv[1:]))\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/script"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--star"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/star_alignment_report"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--trim1"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/trimgalore_report_fastq_1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--trim2"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/trimgalore_report_fastq_2"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(get_output_prefix()+\".md\")"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/collected_statistics_md"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(get_output_prefix()+\".tsv\")"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/collected_statistics_tsv"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(get_output_prefix()+\".yaml\")"
                    },
                    "id": "#collect-statistics-rna-quantseq.cwl/collected_statistics_yaml"
                }
            ],
            "baseCommand": [
                "python",
                "-c"
            ],
            "doc": "Tool processes and combines log files generated by Trimgalore, Bowtie, Samtools and MACS2.\n\n`get_output_prefix` function returns output file prefix equal to `output_prefix`+`_collected_statistics_report` (if this input is provided) or\ngenerated on the base of bowtie log basename with `_collected_statistics_report` extension.\n",
            "id": "#collect-statistics-rna-quantseq.cwl",
            "http://schema.org/name": "collect-statistics-rna-seq",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/collect-statistics-rna-seq.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Runs python code from the script input\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.3"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 6
                    },
                    "doc": "Compressed or uncompressed FASTQ file(s)\n",
                    "id": "#extract-fastq.cwl/compressed_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nshopt -s nocaseglob\nset -- \"$0\" \"$@\"\n\nfunction extract {\n  FILE=$1\n  COMBINED=$2\n  T=`file -b \"${FILE}\" | awk '{print $1}'`\n  case \"${T}\" in\n    \"bzip2\"|\"gzip\"|\"Zip\")\n      7z e -so \"${FILE}\" >> \"${COMBINED}\"\n      ;;\n    \"ASCII\")\n      cat \"${FILE}\" >> \"${COMBINED}\" || true\n      ;;\n    *)\n      echo \"Error: file type unknown\"\n      rm -f \"${COMBINED}\"\n      exit 1\n  esac\n} \n\nCOMBINED=\"\"\nfor FILE in \"$@\"; do\n    BASENAME=$(basename \"$FILE\")\n    ROOTNAME=\"${BASENAME%.*}\"\n    EXT_LIST=( \".fastq\" \".fq\" )\n    for ITEM in $EXT_LIST; do\n      if [[ $ROOTNAME == *$ITEM ]]; then\n        ROOTNAME=\"${ROOTNAME%.*}\"\n      fi\n    done\n    if [[ $COMBINED == \"\" ]]; then\n      COMBINED=\"${ROOTNAME}\"\n    else\n      COMBINED=\"${COMBINED}\"_\"${ROOTNAME}\"\n    fi\ndone\nCOMBINED=\"${COMBINED}\".fastq\n\nfor FILE in \"$@\"; do\n    echo \"Extracting:\" $FILE;\n    extract \"${FILE}\" \"${COMBINED}\"\ndone;\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Bash script to extract compressed FASTQ file\n",
                    "id": "#extract-fastq.cwl/script"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*"
                    },
                    "id": "#extract-fastq.cwl/fastq_file"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Tool to decompress input FASTQ file(s).\nIf several FASTQ files are provided, they will be concatenated in the order that corresponds to files in input.\nBash script's logic:\n- disable case sensitive glob check\n- check if root name of input file already include '.fastq' or '.fq' extension. If yes, set DEFAULT_EXT to \"\",\n  otherwise use '.fastq'\n- check file type, decompress if needed\n- return 1, if file type is not recognized\nThis script also works of input file doesn't have any extension at all\n",
            "id": "#extract-fastq.cwl",
            "http://schema.org/name": "extract-fastq",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/extract-fastq.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Tool to decompress input FASTQ file"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.') + \".fastxstat\" };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/fastx_toolkit:v0.0.14"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-i"
                    },
                    "doc": "FASTA/Q input file. If FASTA file is given, only nucleotides distribution is calculated (there's no quality info)\n",
                    "id": "#fastx-quality-stats.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-N"
                    },
                    "doc": "New output format (with more information per nucleotide/cycle).\ncycle (previously called 'column') = cycle number\nmax-count\nFor each nucleotide in the cycle (ALL/A/C/G/T/N):\n    count   = number of bases found in this column.\n    min     = Lowest quality score value found in this column.\n    max     = Highest quality score value found in this column.\n    sum     = Sum of quality score values for this column.\n    mean    = Mean quality score value for this column.\n    Q1\t= 1st quartile quality score.\n    med\t= Median quality score.\n    Q3\t= 3rd quartile quality score.\n    IQR\t= Inter-Quartile range (Q3-Q1).\n    lW\t= 'Left-Whisker' value (for boxplotting).\n    rW\t= 'Right-Whisker' value (for boxplotting).\n",
                    "id": "#fastx-quality-stats.cwl/new_output_format"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-o",
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Output file to store generated statistics. If not provided - return from default_output_filename function\n",
                    "id": "#fastx-quality-stats.cwl/output_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\"){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "doc": "Generated statistics file",
                    "id": "#fastx-quality-stats.cwl/statistics_file"
                }
            ],
            "baseCommand": [
                "fastx_quality_stats"
            ],
            "doc": "Tool calculates statistics on the base of FASTQ file quality scores.\nIf `output_filename` is not provided call function `default_output_filename` to return default output file name\ngenerated as `input_file` basename + `.fastxstat` extension.\n",
            "id": "#fastx-quality-stats.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#fastx-toolkit-metadata.yaml",
                "http://schema.org/name": "fastx-toolkit",
                "http://schema.org/about": "The FASTX-Toolkit is a collection of command line tools for Short-Reads FASTA/FASTQ files preprocessing. Available Tools:\n  FASTQ-to-FASTA converter (Convert FASTQ files to FASTA files)\n  FASTQ Information (Chart Quality Statistics and Nucleotide Distribution)\n  FASTQ/A Collapser (Collapsing identical sequences in a FASTQ/A file into a single sequence)\n  FASTQ/A Trimmer (Shortening reads in a FASTQ or FASTQ files, removing barcodes or noise)\n  FASTQ/A Renamer (Renames the sequence identifiers in FASTQ/A file)\n  FASTQ/A Clipper (Removing sequencing adapters/linkers)\n  FASTQ/A Reverse-Complement (Producing the Reverse-complement of each sequence in a FASTQ/FASTA file)\n  FASTQ/A Barcode splitter (Splitting a FASTQ/FASTA files containning multiple samples)\n  FASTA Formatter (changes the width of sequences line in a FASTA file)\n  FASTA Nucleotide Changer (Convets FASTA sequences from/to RNA/DNA)\n  FASTQ Quality Filter (Filters sequences based on quality)\n  FASTQ Quality Trimmer (Trims (cuts) sequences based on quality)\n  FASTQ Masker (Masks nucleotides with 'N' (or other character) based on quality)\n",
                "http://schema.org/url": "http://hannonlab.cshl.edu/fastx_toolkit/index.html",
                "http://schema.org/codeRepository": "https://github.com/agordon/fastx_toolkit",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "0.0.14",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C",
                "http://schema.org/discussionUrl": [
                    "https://github.com/agordon/fastx_toolkit/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "The Hannon Lab",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Assaf Gordon",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "fastx-quality-stats",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/fastx-quality-stats.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage: fastx_quality_stats [-h] [-N] [-i INFILE] [-o OUTFILE]\nPart of FASTX Toolkit 0.0.14 by A. Gordon (assafgordon@gmail.com)\n   [-h] = This helpful help screen.\n   [-i INFILE]  = FASTQ input file. default is STDIN.\n   [-o OUTFILE] = TEXT output file. default is STDOUT.\n   [-N]         = New output format (with more information per nucleotide/cycle).\nThe *OLD* output TEXT file will have the following fields (one row per column):\n\tcolumn\t= column number (1 to 36 for a 36-cycles read solexa file)\n\tcount   = number of bases found in this column.\n\tmin     = Lowest quality score value found in this column.\n\tmax     = Highest quality score value found in this column.\n\tsum     = Sum of quality score values for this column.\n\tmean    = Mean quality score value for this column.\n\tQ1\t= 1st quartile quality score.\n\tmed\t= Median quality score.\n\tQ3\t= 3rd quartile quality score.\n\tIQR\t= Inter-Quartile range (Q3-Q1).\n\tlW\t= 'Left-Whisker' value (for boxplotting).\n\trW\t= 'Right-Whisker' value (for boxplotting).\n\tA_Count\t= Count of 'A' nucleotides found in this column.\n\tC_Count\t= Count of 'C' nucleotides found in this column.\n\tG_Count\t= Count of 'G' nucleotides found in this column.\n\tT_Count\t= Count of 'T' nucleotides found in this column.\n\tN_Count = Count of 'N' nucleotides found in this column.\n\tmax-count = max. number of bases (in all cycles)\nThe *NEW* output format:\n\tcycle (previously called 'column') = cycle number\n\tmax-count\n  For each nucleotide in the cycle (ALL/A/C/G/T/N):\n\t\tcount   = number of bases found in this column.\n\t\tmin     = Lowest quality score value found in this column.\n\t\tmax     = Highest quality score value found in this column.\n\t\tsum     = Sum of quality score values for this column.\n\t\tmean    = Mean quality score value for this column.\n\t\tQ1\t= 1st quartile quality score.\n\t\tmed\t= Median quality score.\n\t\tQ3\t= 3rd quartile quality score.\n\t\tIQR\t= Inter-Quartile range (Q3-Q1).\n\t\tlW\t= 'Left-Whisker' value (for boxplotting).\n\t\trW\t= 'Right-Whisker' value (for boxplotting).\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_prefix = function() { var ext = '.'; var root = inputs.bam_file.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.bam_file.basename+ext:root+ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/geep:v0.0.5"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "File"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--annotation"
                    },
                    "doc": "Set the path to the GTF or TAB-delimited file. Required\n",
                    "id": "#geep.cwl/annotation_file"
                },
                {
                    "type": [
                        "File"
                    ],
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--bam"
                    },
                    "doc": "Set the path to the coordinate sorted BAM file. Required\n",
                    "id": "#geep.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 20,
                        "prefix": "--dutp"
                    },
                    "doc": "Set this dutp flag if strand specific analysys should be made. Default: False\n",
                    "id": "#geep.cwl/dutp"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 16,
                        "prefix": "--exclude"
                    },
                    "doc": "Coma separated list of chromosomes to be ignored. Default: \"\"\n",
                    "id": "#geep.cwl/exclude_chr"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 19,
                        "prefix": "--keepUnique"
                    },
                    "doc": "Set this flag if you want prevent distributing the isoform unique reads among other isoforms. Default: False\n",
                    "id": "#geep.cwl/keep_unique"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--log"
                    },
                    "doc": "Set the path to save LOG file. Default: /dev/null\n",
                    "id": "#geep.cwl/log_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 21,
                        "prefix": "--cycles"
                    },
                    "doc": "Set the maximum number of cycles used for read balancing. Default: 2000\n",
                    "id": "#geep.cwl/max_cycles"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 17,
                        "prefix": "--minIntLen"
                    },
                    "doc": "Set the minimal interval length. All shorter intervals will be discarded. Default: 0\n",
                    "id": "#geep.cwl/min_interval_length"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 18,
                        "prefix": "--minReadLen"
                    },
                    "doc": "Set the minimal read length. All parts of spliced reads that intersect with exon in\nless than minReadLen nucleotides will be discarded. Default: 0\n",
                    "id": "#geep.cwl/min_read_length"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--output",
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_prefix();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Set the prefix to save output files. Default: \"\"\n",
                    "id": "#geep.cwl/output_prefix"
                },
                {
                    "type": [
                        "null",
                        "double"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "--cutoff"
                    },
                    "doc": "Set rpkm cutoff value to be used for all rpkms below --threshold. Default: 0\n",
                    "id": "#geep.cwl/rpkm_cutoff"
                },
                {
                    "type": [
                        "null",
                        "double"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "--threshold"
                    },
                    "doc": "Set rpkm cutoff threshold, below which everything will be changed to value set with --cutoff. Default: 0\n",
                    "id": "#geep.cwl/rpkm_threshold"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 22,
                        "prefix": "--threads"
                    },
                    "doc": "Set the number of threads. Default: 1\n",
                    "id": "#geep.cwl/threads"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*genes.csv"
                    },
                    "id": "#geep.cwl/genes_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*isoforms.csv"
                    },
                    "id": "#geep.cwl/isoforms_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.log_filename)"
                    },
                    "id": "#geep.cwl/log_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*raw.txt"
                    },
                    "id": "#geep.cwl/raw_file"
                }
            ],
            "baseCommand": [
                "geep"
            ],
            "label": "geep",
            "doc": "Tool calculates RPKM values grouped by isoforms or genes.\n\n`default_output_prefix` function returns default prefix based on `bam_file` basename, if `output_prefix` is not\nprovided.\n",
            "id": "#geep.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#geep-metadata.yaml",
                "http://schema.org/name": "geep",
                "http://schema.org/about": "Program to calculate RPKM values from BAM and annotation files.\nResults are grouped by isoform and gene names.\n",
                "http://schema.org/url": "https://github.com/Barski-lab/geep/blob/master/README.md",
                "http://schema.org/codeRepository": "https://github.com/Barski-lab/geep",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "0.0.2",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/discussionUrl": [
                    "https://github.com/Barski-lab/geep/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Michael Kotliar",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "geep",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/geep.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\n  geep [params]\n    -b,--bam         Set the path to the BAM file. Required\n    -g,--annotation  Set the path to the GTF or TAB-delimited file. Required\n    -l,--log         Set the path to save LOG file. Default: /dev/null\n    -o,--output      Set the prefix to save output files. Default: \"\"\n    -t,--threshold   Set rpkm cutoff threshold, below which everything will be changed to value set with --cutoff. Default: 0\n    -c,--cutoff      Set rpkm cutoff value to be used for all rpkms below --threshold. Default: 0\n    -x,--exclude     Coma separated list of chromosomes to be ignored. Default: \"\"\n    -i,--minIntLen   Set the minimal interval length. All shorter intervals will be discarded. Default: 0\n    -r,--minReadLen  Set the minimal read length. All parts of spliced reads that intersect with exon in less than minReadLen nucleotides will be discarded. Default: 0\n    -p,--threads     Set the number of threads. Default: 1\n    -u,--keepUnique  Set this flag if you want prevent distributing the isoform unique reads among other isoforms. Default: False\n    -d,--dutp        Set this dutp flag if strand specific analysys should be made. Default: False\n    -c,--cycles      Set the maximum number of cycles used for read balancing. Default: 2000"
        },
        {
            "class": "CommandLineTool",
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/htseq:0.13.5--py38h1773678_0"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#htseq-count.cwl/additional_id/gene_id",
                                "#htseq-count.cwl/additional_id/transcript_id",
                                "#htseq-count.cwl/additional_id/exon_number",
                                "#htseq-count.cwl/additional_id/exon_id",
                                "#htseq-count.cwl/additional_id/gene_name"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--additional-attr"
                    },
                    "doc": "Additional feature attributes, which will be printed as an additional\ncolumn after the primary attribute column but before the counts column.\nDefault: none\n",
                    "id": "#htseq-count.cwl/additional_id"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Path to the coordinate sorted indexed BAM file\n",
                    "id": "#htseq-count.cwl/alignment_bam_file"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 6
                    },
                    "doc": "GTF annotation file\n",
                    "id": "#htseq-count.cwl/annotation_gtf_file"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#htseq-count.cwl/feature_id/gene_id",
                                "#htseq-count.cwl/feature_id/transcript_id",
                                "#htseq-count.cwl/feature_id/exon_id",
                                "#htseq-count.cwl/feature_id/gene_name"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "-i"
                    },
                    "doc": "GTF attribute to be used as feature ID. Several GTF lines with the same feature\nID will be considered as parts of the same feature. The feature ID is used to\nidentity the counts in the output table.\nFYI: no use of having here \"exon_number\", so it was excluded from enum\nDefault: gene_id\n",
                    "id": "#htseq-count.cwl/feature_id"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#htseq-count.cwl/feature_type/CDS",
                                "#htseq-count.cwl/feature_type/exon",
                                "#htseq-count.cwl/feature_type/start_codon",
                                "#htseq-count.cwl/feature_type/stop_codon",
                                "#htseq-count.cwl/feature_type/transcript"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "-t"
                    },
                    "doc": "Feature type (3rd column in GFF file) to be used,\nall features of other type are ignored.\nDefault: exon\n",
                    "id": "#htseq-count.cwl/feature_type"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#htseq-count.cwl/nonunique_mode/none",
                                "#htseq-count.cwl/nonunique_mode/all",
                                "#htseq-count.cwl/nonunique_mode/fraction",
                                "#htseq-count.cwl/nonunique_mode/random"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--nonunique"
                    },
                    "doc": "Mode to handle reads that align to or are assigned to more than\none feature in the selected overlapping_mode\nOptions:\n  \"none\": the read (or read pair) is counted as ambiguous and not\n    counted for any features. Also, if the read (or read pair) aligns\n    to more than one location in the reference, it is scored as\n    alignment_not_unique.\n  \"all\": the read (or read pair) is counted as ambiguous and is also\n    counted in all features to which it was assigned. Also, if the read\n    (or read pair) aligns to more than one location in the reference, it\n    is scored as alignment_not_unique and also separately for each location.\n  \"fraction\": the read (or read pair) is counted as ambiguous and is also\n    counted fractionally in all features to which it was assigned. For\n    example, if the read overlaps with 3 features, it will be counted 1/3\n    to each of them.\n  \"random\": the read (or read pair) is counted as ambiguous and is also\n    counted uniformly at random to one of the features to which it\n    was assigned.\nDefault: \"none\"\n",
                    "id": "#htseq-count.cwl/nonunique_mode"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#htseq-count.cwl/overlapping_mode/union",
                                "#htseq-count.cwl/overlapping_mode/intersection-strict",
                                "#htseq-count.cwl/overlapping_mode/intersection-nonempty"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-m"
                    },
                    "doc": "Mode to handle reads overlapping more than one feature\nDefault: union\n",
                    "id": "#htseq-count.cwl/overlapping_mode"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nshopt -s nocaseglob\nset -- \"$0\" \"$@\"\necho \"Run htseq-count with the following parameters\"\nfor i in \"$@\";\n  do echo $i;\ndone;\nhtseq-count -f bam -r pos \"$@\" 1> temp.tsv 2> feature_counts_stderr.log\ncat temp.tsv | grep \"^__\" > feature_counts_stdout.log\ncat temp.tsv | grep -v \"^__\" > feature_counts_report.tsv\nrm -f temp.tsv\n",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Script runs htseq-count with the provided parameters splitting\nthe report file into summary and the actual counts data.\n",
                    "id": "#htseq-count.cwl/script"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#htseq-count.cwl/secondary_alignments_mode/score",
                                "#htseq-count.cwl/secondary_alignments_mode/ignore"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--secondary-alignments"
                    },
                    "doc": "Mode to handle secondary alignments (SAM flag 0x100).\nCan be \"score\" and \"ignore\"\nDefault: \"score\"\n",
                    "id": "#htseq-count.cwl/secondary_alignments_mode"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#htseq-count.cwl/strand_specific/yes",
                                "#htseq-count.cwl/strand_specific/no",
                                "#htseq-count.cwl/strand_specific/reverse"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-s"
                    },
                    "doc": "Whether the data is from a strand-specific assay. For stranded=no, a read is\nconsidered overlapping with a feature regardless of whether it is mapped to\nthe same or the opposite strand as the feature. For stranded=yes and single-end\nreads, the read has to be mapped to the same strand as the feature. For paired-end\nreads, the first read has to be on the same strand and the second read on the\nopposite strand. For stranded=reverse, these rules are reversed.\nDefault: \"yes\"\n",
                    "id": "#htseq-count.cwl/strand_specific"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#htseq-count.cwl/supplementary_alignments_mode/score",
                                "#htseq-count.cwl/supplementary_alignments_mode/ignore"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "--supplementary-alignments"
                    },
                    "doc": "Mode to handle supplementary/chimeric alignments (SAM flag 0x800).\nCan be \"score\" and \"ignore\"\nDefault: \"score\"\n",
                    "id": "#htseq-count.cwl/supplementary_alignments_mode"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "feature_counts_report.tsv"
                    },
                    "id": "#htseq-count.cwl/feature_counts_report_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "feature_counts_stderr.log"
                    },
                    "id": "#htseq-count.cwl/stderr_log"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "feature_counts_stdout.log"
                    },
                    "id": "#htseq-count.cwl/stdout_log"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "label": "HTSeq: Analysing high-throughput sequencing data",
            "doc": "For convenience to use in the workflow that sort and index BAM files by coordinate\nthis tools expects coordinate sorted and indexed BAM file as input. For single-read\ndat it won't influence on anything, for paired-end the more memory will be used to\nkeep reads while looking for their proper pairs (see --max-reads-in-buffer parameter).\n\nCurrent limitations:\n  - only one `--additional-attr` is supported\n  - skip `--nprocesses` parameter as it's not helpful when we use only one input BAM file\n",
            "id": "#htseq-count.cwl",
            "http://schema.org/name": "HTSeq: Analysing high-throughput sequencing data",
            "http://schema.org/alternateName": "HTSeq: Analysing high-throughput sequencing data",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/htseq-count.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage: htseq-count [options] alignment_file gff_file\n\nThis script takes one or more alignment files in SAM/BAM format and a feature file\nin GFF format and calculates for each feature the number of reads mapping to it. See\nhttp://htseq.readthedocs.io/en/master/count.html for details.\n\noptions:\n  -f <format>, --format=<format>\u00b6\n    Format of the input data. Possible values are sam (for text SAM files) and bam\n    (for binary BAM files). Default is sam.\n\n    DEPRECATED: Modern versions of samtools/htslibs, which HTSeq uses to access\n    SAM/BAM/CRAM files, have automatic file type detection. This flag will be removed\n    in future versions of htseq-count.\n\n  -r <order>, --order=<order>\n    For paired-end data, the alignment have to be sorted either by read name or by\n    alignment position. If your data is not sorted, use the samtools sort function\n    of samtools to sort it. Use this option, with name or pos for <order> to indicate\n    how the input data has been sorted. The default is name.\n\n    If name is indicated, htseq-count expects all the alignments for the reads of a\n    given read pair to appear in adjacent records in the input data. For pos, this\n    is not expected; rather, read alignments whose mate alignment have not yet been\n    seen are kept in a buffer in memory until the mate is found. While, strictly speaking,\n    the latter will also work with unsorted data, sorting ensures that most alignment\n    mates appear close to each other in the data and hence the buffer is much less likely\n    to overflow.\n\n  --max-reads-in-buffer=<number>\n    When <alignment_file> is paired end sorted by position, allow only so many reads to\n    stay in memory until the mates are found (raising this number will use more memory).\n    Has no effect for single end or paired end sorted by name. (default: 30000000)\n\n  -s <yes/no/reverse>, --stranded=<yes/no/reverse>\n    Whether the data is from a strand-specific assay (default: yes)\n    For stranded=no, a read is considered overlapping with a feature regardless of whether\n    it is mapped to the same or the opposite strand as the feature. For stranded=yes and\n    single-end reads, the read has to be mapped to the same strand as the feature. For\n    paired-end reads, the first read has to be on the same strand and the second read on\n    the opposite strand. For stranded=reverse, these rules are reversed.\n\n  -a <minaqual>, --a=<minaqual>\n    Skip all reads with MAPQ alignment quality lower than the given minimum value\n    (default: 10). MAPQ is the 5th column of a SAM/BAM file and its usage depends on the\n    software used to map the reads.\n\n  -t <feature type>, --type=<feature type>\n    Feature type (3rd column in GTF file) to be used, all features of other type are ignored\n    (default, suitable for RNA-Seq analysis using an Ensembl GTF file: exon)\n\n  -i <id attribute>, --idattr=<id attribute>\n    GTF attribute to be used as feature ID. Several GTF lines with the same feature ID will\n    be considered as parts of the same feature. The feature ID is used to identity the counts\n    in the output table. The default, suitable for RNA-Seq analysis using an Ensembl GTF file,\n    is gene_id.\n\n  --additional-attr=<id attributes>\n    Additional feature attributes, which will be printed as an additional column after the\n    primary attribute column but before the counts column(s). The default is none, a suitable\n    value to get gene names using an Ensembl GTF file is gene_name. To use more than one\n    additional attribute, repeat the option in the command line more than once, with a single\n    attribute each time, e.g. --additional-attr=gene_name --additional_attr=exon_number.\n\n  -m <mode>, --mode=<mode>\n    Mode to handle reads overlapping more than one feature. Possible values for <mode> are union,\n    intersection-strict and intersection-nonempty (default: union)\n\n  --nonunique=<nonunique mode>\n    Mode to handle reads that align to or are assigned to more than one feature in the overlap\n    <mode> of choice (see -m option). <nonunique mode> are none and all (default: none)\n\n  --secondary-alignments=<mode>\n    Mode to handle secondary alignments (SAM flag 0x100). <mode> can be score and ignore\n    (default: score)\n\n  --supplementary-alignments=<mode>\n    Mode to handle supplementary/chimeric alignments (SAM flag 0x800). <mode> can be score\n    and ignore (default: score)\n\n  -o <samout>, --samout=<samout>\n    Write out all SAM alignment records into SAM files (one per input file needed), annotating\n    each line with its feature assignment (as an optional field with tag \u2018XF\u2019)\n\n  -n <n>, --nprocesses=<n>\n    Number of parallel CPU processes to use (default: 1). This option is useful to process\n    several input files at once. Each file will use only 1 CPU. It is possible, of course, to\n    split a very large input SAM/BAM files into smaller chunks upstream to make use of this option.\n\n  -p <samout_format>, --samout-format=<samout_format>\n    Format to use with the \u2013samout option, can be bam or sam (default: sam).\n\n  -q, --quiet\n    Suppress progress report and warnings\n\n  -h, --help\n    Show a usage summary and exit\n\n  --version\n    Show software version and exit\n\nWritten by Simon Anders (sanders@fs.tum.de), European Molecular Biology\nLaboratory (EMBL). (c) 2010. Released under the terms of the GNU General\nPublic License v3. Part of the 'HTSeq' framework, version 0.11.2."
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_output_filename = function() { if (inputs.output_filename) { return inputs.output_filename; } return inputs.unsorted_file.location.split('/').slice(-1)[0]; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.2"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-k"
                        }
                    },
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "-k, --key=POS1[,POS2]\nstart a key at POS1, end it at POS2 (origin 1)\n",
                    "id": "#linux-sort.cwl/key"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Name for generated output file\n",
                    "id": "#linux-sort.cwl/output_filename"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "id": "#linux-sort.cwl/unsorted_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "id": "#linux-sort.cwl/sorted_file",
                    "outputBinding": {
                        "glob": "$(get_output_filename())"
                    }
                }
            ],
            "stdout": "$(get_output_filename())",
            "baseCommand": [
                "sort"
            ],
            "doc": "Tool sorts data from `unsorted_file` by key\n\n`default_output_filename` function returns file name identical to `unsorted_file`, if `output_filename` is not provided.\n",
            "id": "#linux-sort.cwl",
            "http://schema.org/name": "linux-sort",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/linux-sort.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/plugin-plot-rna:v0.0.4"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--annotation"
                    },
                    "doc": "Path to the annotation TSV/CSV file\n",
                    "id": "#plugin-plot-rna.cwl/annotation_file"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--bam"
                    },
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "doc": "Path to the indexed BAM file\n",
                    "id": "#plugin-plot-rna.cwl/bambai_pair"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--isoforms"
                    },
                    "doc": "Path to the isoforms TSV/CSV file\n",
                    "id": "#plugin-plot-rna.cwl/isoforms_file"
                },
                {
                    "type": "int",
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--mapped"
                    },
                    "doc": "Mapped reads number\n",
                    "id": "#plugin-plot-rna.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--minlength"
                    },
                    "doc": "Ignore isoforms shorter than --minlength.\nDefault: 1000\n",
                    "id": "#plugin-plot-rna.cwl/minimum_isoform_length"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--minrpkm"
                    },
                    "doc": "Ignore isoforms with RPKM smaller than --minrpkm.\nDefault: 10\n",
                    "id": "#plugin-plot-rna.cwl/minimum_rpkm"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "--output"
                    },
                    "doc": "Output prefix. Default: ./coverage\n",
                    "id": "#plugin-plot-rna.cwl/output_prefix"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--pair"
                    },
                    "doc": "Run as paired end. Default: false\n",
                    "id": "#plugin-plot-rna.cwl/pair"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#plugin-plot-rna.cwl/strand_specificity/yes",
                                "#plugin-plot-rna.cwl/strand_specificity/no",
                                "#plugin-plot-rna.cwl/strand_specificity/reverse"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--stranded"
                    },
                    "doc": "Whether the data is from a strand-specific assay.\n--stranded no      - a read is considered overlapping with a feature regardless of whether\n                     it is mapped to the same or the opposite strand as the feature.\n--stranded yes     - the read has to be mapped to the same strand as the feature.\n--stranded reverse - the read has to be mapped to the opposite strand than the feature.\n",
                    "id": "#plugin-plot-rna.cwl/strand_specificity"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "--threads"
                    },
                    "doc": "Threads. Default: 1\n",
                    "id": "#plugin-plot-rna.cwl/threads"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*gene_body_plot.pdf"
                    },
                    "id": "#plugin-plot-rna.cwl/gene_body_plot_pdf"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*gene_body_plot.png"
                    },
                    "id": "#plugin-plot-rna.cwl/gene_body_plot_png"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*gene_body_report.tsv"
                    },
                    "id": "#plugin-plot-rna.cwl/gene_body_report_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*rpkm_distribution_plot.pdf"
                    },
                    "id": "#plugin-plot-rna.cwl/rpkm_distribution_plot_pdf"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*rpkm_distribution_plot.png"
                    },
                    "id": "#plugin-plot-rna.cwl/rpkm_distribution_plot_png"
                },
                {
                    "type": "File",
                    "id": "#plugin-plot-rna.cwl/stderr_log",
                    "outputBinding": {
                        "glob": "gene_body_stderr.log"
                    }
                },
                {
                    "type": "File",
                    "id": "#plugin-plot-rna.cwl/stdout_log",
                    "outputBinding": {
                        "glob": "gene_body_stdout.log"
                    }
                }
            ],
            "baseCommand": [
                "plot_rna.R"
            ],
            "successCodes": [
                1
            ],
            "stderr": "gene_body_stderr.log",
            "stdout": "gene_body_stdout.log",
            "label": "Gene body average tag density plot and RPKM distribution histogram",
            "doc": "Runs R script to produce gene body average tag density plot and RPKM distribution histogram\nDoesn't fail even when we couldn't produce any plots\n",
            "id": "#plugin-plot-rna.cwl",
            "http://schema.org/name": "Gene body average tag density plot and RPKM distribution histogram",
            "http://schema.org/alternateName": "Gene body average tag density plot and RPKM distribution histogram",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/plugin-plot-rna.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage: plugin_plot_rna.R\n      [-h] --annotation ANNOTATION --bam BAM --isoforms ISOFORMS\n      [--minrpkm MINRPKM] [--minlength MINLENGTH] --mapped MAPPED [--pair]\n      [--stranded {yes,no,reverse}] [--output OUTPUT] [--threads THREADS]\n\nGene body average tag density plot and RPKM distribution histogram for\nisoforms\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --annotation ANNOTATION\n                        Path to the annotation TSV/CSV file with gene names\n                        set in name2 field\n  --bam BAM             Path to the indexed BAM file\n  --isoforms ISOFORMS   Path to the RPKM isoforms expression TSV/CSV file\n  --minrpkm MINRPKM     Ignore isoforms with RPKM smaller than --minrpkm.\n                        Default: 10\n  --minlength MINLENGTH\n                        Ignore isoforms shorter than --minlength. Default:\n                        1000\n  --mapped MAPPED       Mapped reads/pairs number\n  --pair                Run as paired end. Default: false\n  --stranded {yes,no,reverse}\n                        Strand specificity. One of \"yes\", \"no\" or \"reverse\".\n                        Default: \"no\"\n  --output OUTPUT       Output prefix. Default: ./coverage\n  --threads THREADS     Threads. Default: 1"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_target_name = function() { return inputs.target_filename.split('/').slice(-1)[0]; }"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.3"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\ncp $0 $1\nif [ -f $0.bai ]; then\n  cp $0.bai $1.bai\nfi\n",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#rename.cwl/script"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5
                    },
                    "id": "#rename.cwl/source_file"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 6,
                        "valueFrom": "$(get_target_name())"
                    },
                    "id": "#rename.cwl/target_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(get_target_name())"
                    },
                    "secondaryFiles": "${\n    if (inputs.source_file.secondaryFiles && inputs.source_file.secondaryFiles.length > 0){\n      return inputs.target_filename+\".bai\";\n    } else {\n      return \"null\";\n    }\n  }\n",
                    "id": "#rename.cwl/target_file"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Tool renames `source_file` to `target_filename`.\nInput `target_filename` should be set as string. If it's a full path, only basename will be used.\nIf BAI file is present, it will be renamed too\n",
            "id": "#rename.cwl",
            "http://schema.org/name": "rename",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/rename.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "cp source target"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var ext = function() { if (inputs.csi && !inputs.bai){ return '.csi'; } else { return '.bai'; } };",
                        "var default_bam = function() { if (inputs.trigger == true){ return inputs.sort_input.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".bam\"; } else { return inputs.sort_input.location.split('/').slice(-1)[0]; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/samtools:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Generate BAI-format index for BAM files [default]. If input isn't cram.\n",
                    "id": "#samtools-sort-index.cwl/bai"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: samtools index \" ${@:1}\n  samtools index \"${@:1}\"\nelse\n  echo \"Skip samtools index \" ${@:1}\nfi\n",
                    "inputBinding": {
                        "position": 20
                    },
                    "doc": "Bash function to run samtools index with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-sort-index.cwl/bash_script_index"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: samtools sort \" ${@:1}\n  samtools sort \"${@:1}\"\nelse\n  echo \"Skip samtools sort \" ${@:1}\n  SOURCE=${@:(-1):1}\n  TARGET=$(basename \"$SOURCE\")\n  cp $SOURCE $TARGET\nfi\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Bash function to run samtools sort with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-sort-index.cwl/bash_script_sort"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Generate CSI-format index for BAM files. If input isn't cram.\n",
                    "id": "#samtools-sort-index.cwl/csi"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 24,
                        "prefix": "-m"
                    },
                    "doc": "Set minimum interval size for CSI indices to 2^INT [14]\n",
                    "id": "#samtools-sort-index.cwl/csi_interval"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-l"
                    },
                    "doc": "SORT: desired compression level for the final output file, ranging from 0 (uncompressed)\nor 1 (fastest but minimal compression) to 9 (best compression but slowest to write),\nsimilarly to gzip(1)'s compression level setting.\nIf -l is not used, the default compression level will apply.\n",
                    "id": "#samtools-sort-index.cwl/sort_compression_level"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16
                    },
                    "doc": "Input only in.sam|in.bam|in.cram. Optionally could be supplemented with index file in secondaryFiles\n",
                    "id": "#samtools-sort-index.cwl/sort_input"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-o",
                        "valueFrom": "${\n    if (self == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Write the final sorted output to FILE. Only out.bam|out.cram.\nIf output file extension is set to SAM, tool will fail on the index step\n",
                    "id": "#samtools-sort-index.cwl/sort_output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Set number of sorting and compression threads [1] (Only for sorting)\n",
                    "id": "#samtools-sort-index.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "doc": "If true - run samtools, if false - return sort_input and optional index file in secondaryFiles.\n",
                    "id": "#samtools-sort-index.cwl/trigger"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return inputs.sort_output_filename;\n    }\n}\n"
                    },
                    "secondaryFiles": "${ if (inputs.trigger == true){ return self.basename + ext(); } else { return inputs.sort_input.secondaryFiles?inputs.sort_input.secondaryFiles:\"null\"; } }",
                    "id": "#samtools-sort-index.cwl/bam_bai_pair"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "arguments": [
                {
                    "valueFrom": "${ return inputs.trigger ? \"true\" : \"false\" }\n",
                    "position": 6
                },
                {
                    "valueFrom": "bam",
                    "position": 13,
                    "prefix": "-O"
                },
                {
                    "valueFrom": "$(inputs.threads?inputs.threads:1)",
                    "position": 15,
                    "prefix": "-@"
                },
                {
                    "valueFrom": ";",
                    "position": 17,
                    "shellQuote": false
                },
                {
                    "valueFrom": "bash",
                    "position": 18
                },
                {
                    "valueFrom": "-c",
                    "position": 19
                },
                {
                    "valueFrom": "${ return inputs.trigger ? \"true\" : \"false\" }\n",
                    "position": 21
                },
                {
                    "valueFrom": "$(inputs.bai?'-b':inputs.csi?'-c':[])",
                    "position": 23
                },
                {
                    "valueFrom": "$(inputs.threads?inputs.threads:1)",
                    "position": 25,
                    "prefix": "-@"
                },
                {
                    "valueFrom": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return inputs.sort_output_filename;\n    }\n}\n",
                    "position": 26
                },
                {
                    "valueFrom": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam() + ext();\n    } else {\n      return inputs.sort_output_filename + ext();\n    }\n}\n",
                    "position": 27
                }
            ],
            "doc": "Tool to sort and index input BAM/SAM/CRAM.\nIf input `trigger` is set to `true` or isn't set at all (`true` is used by default), run `samtools sort` and\n`samtools index`, return sorted BAM and BAI/CSI index file.\nIf input `trigger` is set to `false`, return unchanged `sort_input` (BAM/SAM/CRAM) and index (BAI/CSI, if provided in\n`secondaryFiles`) files.\n\nTrigger logic is implemented in two bash scripts set by default as `bash_script_sort` and `bash_script_index` inputs.\nFor both of then, if the first argument $0 (which is `trigger` input) is true, run `samtools sort/index` with the rest\nof the arguments. If $0 is not true, skip `samtools sort/index` and return `sort_input` and `secondaryFiles`\n(if provided).\n\nInput `trigger` is Boolean, but returns String, because of `valueFrom` field. The `valueFrom` is used, because if `trigger`\nis false, cwl-runner doesn't append this argument at all to the the `baseCommand` - new feature of CWL v1.0.2. Alternatively,\n`prefix` field could be used, but it causes changing in script logic.\n\nIf using `sort_output_filename`, the output file extension should be `*.bam`, because `samtools sort` defines the output\nfile format on the base of the file extension. If `*.sam` is sed as output filename, it cannot be usefully indexed\nby `samtools index`.\n\n`default_bam` function is used to generate output filename for `samtools sort` if input `sort_output_filename` is not\nset or when `trigger` is false and we need to return `sort_input` and `secondaryFiles` (if provided) files. Output\nfilename is generated on the base of `sort_input` basename with `.bam` extension by default.\n\n`ext` function is used to return the index file extension (BAI/CSI) based on `csi` and `bai` inputs according to the\nfollowing logic\n  `csi` &&  `bai`  => BAI\n !`csi` && !`bai ` => BAI\n  `csi` && !`bai ` => CSI\n",
            "id": "#samtools-sort-index.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#samtools-metadata.yaml",
                "http://schema.org/name": "samtools",
                "http://schema.org/about": "A suite of programs for interacting with high-throughput sequencing data. It consists of three separate repositories: Samtools (Reading/writing/editing/indexing/viewing SAM/BAM/CRAM format), BCFtools (Reading/writing BCF2/VCF/gVCF files and calling/filtering/summarising SNP and short indel sequence variants) and HTSlib (A C library for reading/writing high-throughput sequencing data).\n",
                "http://schema.org/url": "http://www.htslib.org/",
                "http://schema.org/codeRepository": "https://github.com/samtools/samtools.git",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT",
                    "https://opensource.org/licenses/BSD-3-Clause"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "1.4",
                    "http://schema.org/applicationCategory": "commandline tool"
                },
                "http://schema.org/programmingLanguage": "C, Perl",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btr509",
                        "http://schema.org/name": "(Li, 2011) A statistical framework for SNP calling, mutation discovery, association mapping and population genetical parameter estimation from sequencing data. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/21903627"
                    },
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btr076",
                        "http://schema.org/name": "(Li, 2011) Improving SNP discovery by base alignment quality. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/21320865"
                    },
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btp352",
                        "http://schema.org/name": "(Li et al., 2009) The Sequence Alignment/Map format and SAMtools. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/19505943"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://lists.sourceforge.net/lists/listinfo/samtools-help",
                    "https://lists.sourceforge.net/lists/listinfo/samtools-devel"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Sanger Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Heng Li",
                                "http://schema.org/description": "wrote most of the initial source codes of SAMtools and various converters."
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Broad Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Bob Handsaker",
                                "http://schema.org/description": "A major contributor to the\nSAM/BAM specification. He designed and implemented the BGZF format, the\nunderlying indexable compression format for the BAM format. BGZF does\nnot support arithmetic between file offsets.\n"
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Beijing Genome Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Jue Ruan",
                                "http://schema.org/description": "Designed and implemented the\nRAZF format, an alternative indexable compression format. RAZF is no longer\nused by or provided with SAMtools. Source code remains available in older\nSAMtools 0.1.x releases and from the standalone branch in the repository.\n"
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Person",
                        "http://schema.org/name": "Colin Hercus",
                        "http://schema.org/description": "updated novo2sam.pl to support gapped alignment by novoalign."
                    },
                    {
                        "class": "http://schema.org/Person",
                        "http://schema.org/name": "Petr Danecek",
                        "http://schema.org/description": "contributed the header parsing library sam_header.c and sam2vcf.pl script."
                    }
                ]
            },
            "http://schema.org/name": "samtools-sort-index",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/samtools-sort-index.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: samtools sort [options...] [in.bam]\nOptions:\n  -l INT     Set compression level, from 0 (uncompressed) to 9 (best)\n  -m INT     Set maximum memory per thread; suffix K/M/G recognized [768M]\n  -n         Sort by read name\n  -o FILE    Write final output to FILE rather than standard output\n  -T PREFIX  Write temporary files to PREFIX.nnnn.bam\n      --input-fmt-option OPT[=VAL]\n               Specify a single input file format option in the form\n               of OPTION or OPTION=VALUE\n  -O, --output-fmt FORMAT[,OPT[=VAL]]...\n               Specify output format (SAM, BAM, CRAM)\n      --output-fmt-option OPT[=VAL]\n               Specify a single output file format option in the form\n               of OPTION or OPTION=VALUE\n      --reference FILE\n               Reference sequence FASTA FILE [null]\n  -@, --threads INT\n               Number of additional threads to use [0]\n\nUsage: samtools index [-bc] [-m INT] <in.bam> [out.index]\nOptions:\n  -b       Generate BAI-format index for BAM files [default]\n  -c       Generate CSI-format index for BAM files\n  -m INT   Set minimum interval size for CSI indices to 2^INT [14]\n  -@ INT   Sets the number of threads [none]\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { if (inputs.output_filename == \"\"){ var root = inputs.bambai_pair.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.bambai_pair.basename+\".log\":root+\".log\"; } else { return inputs.output_filename; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/samtools:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5
                    },
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "doc": "Coordinate sorted BAM alignment and index BAI files",
                    "id": "#samtools-stats.cwl/bambai_pair"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "",
                    "doc": "Output file name",
                    "id": "#samtools-stats.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nsamtools stats $0 > all_statistics.log\ncat all_statistics.log | grep SN | cut -f 2- > sn_section.log\necho -e \"insert size\\tpairs total\\tinward oriented pairs\\toutward oriented pairs\\tother pairs\" > is_section.tsv\ncat all_statistics.log | grep ^IS | cut -f 2- | head -n 1501 >> is_section.tsv\ncat all_statistics.log\n",
                    "inputBinding": {
                        "position": 4
                    },
                    "doc": "samtools stats with optional filtering criteria passes as $1",
                    "id": "#samtools-stats.cwl/script"
                }
            ],
            "outputs": [
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "sn_section.log",
                        "outputEval": "${\n  var s = self[0].contents.substring(self[0].contents.indexOf(\"average length\"));\n  return parseInt(s.substring(s.indexOf(\"average length\")+16, s.indexOf(\"\\n\")));\n}\n"
                    },
                    "doc": "Reads Average Length",
                    "id": "#samtools-stats.cwl/average_length"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "is_section.tsv"
                    },
                    "doc": "BAM file statistics (IS section)",
                    "id": "#samtools-stats.cwl/ext_is_section"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(default_output_filename())"
                    },
                    "doc": "BAM file statistics",
                    "id": "#samtools-stats.cwl/log_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "sn_section.log",
                        "outputEval": "${\n  var s = self[0].contents.substring(self[0].contents.indexOf(\"raw total sequences\"));\n  return parseInt(s.substring(s.indexOf(\"raw total sequences\")+21, s.indexOf(\"\\n\")));\n}\n"
                    },
                    "doc": "Raw Total Sequences",
                    "id": "#samtools-stats.cwl/raw_total_sequences"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "sn_section.log",
                        "outputEval": "${\n  var s = self[0].contents.substring(self[0].contents.indexOf(\"reads mapped\"));\n  return parseInt(s.substring(s.indexOf(\"reads mapped\")+14, s.indexOf(\"\\n\")));\n}\n"
                    },
                    "doc": "Reads Mapped",
                    "id": "#samtools-stats.cwl/reads_mapped"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "stdout": "$(default_output_filename())",
            "doc": "Generates statistics for the input BAM file.\n",
            "id": "#samtools-stats.cwl",
            "http://schema.org/mainEntity": {
                "$import": "#samtools-metadata.yaml"
            },
            "http://schema.org/name": "samtools-stats",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/samtools-stats.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Generates statistics for the input BAM file.\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_name_prefix = function() { if (Array.isArray(inputs.readFilesIn) && inputs.readFilesIn.length > 0){ return inputs.readFilesIn[0].location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".\"; } else { return inputs.readFilesIn.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".\"; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "scidap/star:v2.7.5c"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignEndsProtrude"
                    },
                    "doc": "0    ConcordantPair\nint, string:\nallow protrusion of alignment ends, i.e. start (end) of the +strand mate downstream of the start (end) of the -strand mate\n  1st word: int: maximum number of protrusion bases allowed\n  2nd word: string:\n    ConcordantPair ... report alignments with non-zero protrusion as concordant pairs\n    DiscordantPair ... report alignments with non-zero protrusion as discordant pairs\n",
                    "id": "#star-alignreads.cwl/alignEndsProtrude"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignEndsType"
                    },
                    "doc": "Local\nstring: type of read ends alignment\nLocal           ... standard local alignment with soft-clipping allowed\nEndToEnd        ... force end-to-end read alignment, do not soft-clip\nExtend5pOfRead1 ... fully extend only the 5p of the read1, all other ends: local alignment\nExtend5pOfReads12 ... fully extend only the 5p of the both read1 and read2, all other ends: local alignment\n",
                    "id": "#star-alignreads.cwl/alignEndsType"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignIntronMax"
                    },
                    "doc": "0\nmaximum intron size, if 0, max intron size will be determined by (2^winBinNbits)*winAnchorDistNbins\n",
                    "id": "#star-alignreads.cwl/alignIntronMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignIntronMin"
                    },
                    "doc": "21\nminimum intron size: genomic gap is considered intron if its length>=alignIntronMin,\notherwise it is considered Deletion\n",
                    "id": "#star-alignreads.cwl/alignIntronMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignMatesGapMax"
                    },
                    "doc": "0\nmaximum gap between two mates, if 0, max intron gap will be determined by (2^winBinNbits)*winAnchorDistNbins\n",
                    "id": "#star-alignreads.cwl/alignMatesGapMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJDBoverhangMin"
                    },
                    "doc": "3\nint>0: minimum overhang (i.e. block size) for annotated (sjdb) spliced alignments\n",
                    "id": "#star-alignreads.cwl/alignSJDBoverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJoverhangMin"
                    },
                    "doc": "5\nint>0: minimum overhang (i.e. block size) for spliced alignments\n",
                    "id": "#star-alignreads.cwl/alignSJoverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJstitchMismatchNmax"
                    },
                    "doc": "0 -1 0 0\n4*int>=0: maximum number of mismatches for stitching of the splice junctions (-1: no limit).\n(1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif.\n",
                    "id": "#star-alignreads.cwl/alignSJstitchMismatchNmax"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSoftClipAtReferenceEnds"
                    },
                    "doc": "Yes\nstring: allow the soft-clipping of the alignments past the end of the chromosomes\nYes ... allow\nNo  ... prohibit, useful for compatibility with Cufflinks\n",
                    "id": "#star-alignreads.cwl/alignSoftClipAtReferenceEnds"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSplicedMateMapLmin"
                    },
                    "doc": "0\nint>0: minimum mapped length for a read mate that is spliced\n",
                    "id": "#star-alignreads.cwl/alignSplicedMateMapLmin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSplicedMateMapLminOverLmate"
                    },
                    "doc": "0.66\nfloat>0: alignSplicedMateMapLmin normalized to mate length\n",
                    "id": "#star-alignreads.cwl/alignSplicedMateMapLminOverLmate"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignTranscriptsPerReadNmax"
                    },
                    "doc": "10000\nint>0: max number of different alignments per read to consider\n",
                    "id": "#star-alignreads.cwl/alignTranscriptsPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignTranscriptsPerWindowNmax"
                    },
                    "doc": "100\nint>0: max number of transcripts per window\n",
                    "id": "#star-alignreads.cwl/alignTranscriptsPerWindowNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignWindowsPerReadNmax"
                    },
                    "doc": "10000\nint>0: max number of windows per read\n",
                    "id": "#star-alignreads.cwl/alignWindowsPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bamRemoveDuplicatesMate2basesN"
                    },
                    "doc": "int>0: number of bases from the 5' of mate 2 to use in collapsing (e.g. for\nRAMPAGE)\n",
                    "id": "#star-alignreads.cwl/bamRemoveDuplicatesMate2basesN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bamRemoveDuplicatesType"
                    },
                    "doc": "string: mark duplicates in the BAM file, for now only works with (i) sorted BAM feeded with inputBAMfile, and (ii) for paired-end alignments only\n-                       ... no duplicate removal/marking\nUniqueIdentical         ... mark all multimappers, and duplicate unique mappers. The coordinates, FLAG, CIGAR must be identical\nUniqueIdenticalNotMulti  ... mark duplicate unique mappers but not multimappers.\n",
                    "id": "#star-alignreads.cwl/bamRemoveDuplicatesType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimFilter"
                    },
                    "doc": "banGenomicN\nstring(s): different filters for chimeric alignments\nNone ... no filtering\nbanGenomicN ... Ns are not allowed in the genome sequence around the chimeric junction\n",
                    "id": "#star-alignreads.cwl/chimFilter"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimJunctionOverhangMin"
                    },
                    "doc": "20\nint>=0: minimum overhang for a chimeric junction\n",
                    "id": "#star-alignreads.cwl/chimJunctionOverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimMainSegmentMultNmax"
                    },
                    "doc": "int>=1: maximum number of multi-alignments for the main chimeric segment. =1 will prohibit multimapping main segments.\n",
                    "id": "#star-alignreads.cwl/chimMainSegmentMultNmax"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimOutType"
                    },
                    "doc": "string(s): type of chimeric output\n1st word:\nSeparateSAMold  ... output old SAM into separate Chimeric.out.sam file\nWithinBAM       ... output into main aligned BAM files (Aligned.*.bam)\n2nd word:\nWithinBAM HardClip  ... hard-clipping in the CIGAR for supplemental chimeric alignments (defaultif no 2nd word is present)\nWithinBAM SoftClip  ... soft-clipping in the CIGAR for supplemental chimeric alignments\n",
                    "id": "#star-alignreads.cwl/chimOutType"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreDropMax"
                    },
                    "doc": "20\nint>=0: max drop (difference) of chimeric score (the sum of scores of all chimeric\nsegements) from the read length\n",
                    "id": "#star-alignreads.cwl/chimScoreDropMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreJunctionNonGTAG"
                    },
                    "doc": "-1\nint: penalty for a non-GT/AG chimeric junction\n",
                    "id": "#star-alignreads.cwl/chimScoreJunctionNonGTAG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreMin"
                    },
                    "doc": "0\nint>=0: minimum total (summed) score of the chimeric segments\n",
                    "id": "#star-alignreads.cwl/chimScoreMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreSeparation"
                    },
                    "doc": "int>=0: minimum difference (separation) between the best chimeric score and\nthe next one\n",
                    "id": "#star-alignreads.cwl/chimScoreSeparation"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimSegmentMin"
                    },
                    "doc": "0\nint>=0: minimum length of chimeric segment length, if ==0, no chimeric output\n",
                    "id": "#star-alignreads.cwl/chimSegmentMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimSegmentReadGapMax"
                    },
                    "doc": "0\nint>=0: maximum gap in the read sequence between chimeric segments\n",
                    "id": "#star-alignreads.cwl/chimSegmentReadGapMax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAdapterMMp"
                    },
                    "doc": "double(s): max proportion of mismatches for 3p adpater clipping for each\nmate.  If one value is given, it will be assumed the same for both mates.\n",
                    "id": "#star-alignreads.cwl/clip3pAdapterMMp"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAdapterSeq"
                    },
                    "doc": "string(s): adapter sequences to clip from 3p of each mate.  If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-alignreads.cwl/clip3pAdapterSeq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAfterAdapterNbases"
                    },
                    "doc": "int(s): number of bases to clip from 3p of each mate after the adapter\nclipping. If one value is given, it will be assumed the same for both\nmates.\n",
                    "id": "#star-alignreads.cwl/clip3pAfterAdapterNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pNbases"
                    },
                    "doc": "int(s): number(s) of bases to clip from 3p of each mate. If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-alignreads.cwl/clip3pNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip5pNbases"
                    },
                    "doc": "int(s): number(s) of bases to clip from 5p of each mate. If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-alignreads.cwl/clip5pNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeChrBinNbits"
                    },
                    "doc": "int: =log2(chrBin), where chrBin is the size of the bins for genome\nstorage: each chromosome will occupy an integer number of bins\n",
                    "id": "#star-alignreads.cwl/genomeChrBinNbits"
                },
                {
                    "type": "Directory",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeDir"
                    },
                    "doc": "string: path to the directory where genome files are stored\n",
                    "id": "#star-alignreads.cwl/genomeDir"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeLoad"
                    },
                    "doc": "NoSharedMemory\nstring: mode of shared memory usage for the genome files\nLoadAndKeep     ... load genome into shared and keep it in memory after run\nLoadAndRemove   ... load genome into shared but remove it after run\nLoadAndExit     ... load genome into shared memory and exit, keeping the genome in memory for future runs\nRemove          ... do not map anything, just remove loaded genome from memory\nNoSharedMemory  ... do not use shared memory, each job will have its own private copy of the genome\n",
                    "id": "#star-alignreads.cwl/genomeLoad"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeSAindexNbases"
                    },
                    "doc": "int: length (bases) of the SA pre-indexing string. Typically between 10 and\n15. Longer strings will use much more memory, but allow faster searches.\n",
                    "id": "#star-alignreads.cwl/genomeSAindexNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeSAsparseD"
                    },
                    "doc": "int>0: suffux array sparsity, i.e. distance between indices: use bigger\nnumbers to decrease needed RAM at the cost of mapping speed reduction\n",
                    "id": "#star-alignreads.cwl/genomeSAsparseD"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--inputBAMfile"
                    },
                    "doc": "string: path to BAM input file, to be used with --runMode\ninputAlignmentsFromBAM\n",
                    "id": "#star-alignreads.cwl/inputBAMfile"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitBAMsortRAM"
                    },
                    "doc": "int>=0: maximum available RAM for sorting BAM. If =0, it will be set to the\ngenome index size. 0 value can only be used with --genomeLoad\nNoSharedMemory option.\n",
                    "id": "#star-alignreads.cwl/limitBAMsortRAM"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitGenomeGenerateRAM"
                    },
                    "doc": "31000000000\nint>0: maximum available RAM (bytes) for genome generation\n",
                    "id": "#star-alignreads.cwl/limitGenomeGenerateRAM"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitIObufferSize"
                    },
                    "doc": "150000000\nint>0: max available buffers size (bytes) for input/output, per thread\n",
                    "id": "#star-alignreads.cwl/limitIObufferSize"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSAMoneReadBytes"
                    },
                    "doc": "100000\nint>0: max size of the SAM record for one read. Recommended value: >(2*(LengthMate1+LengthMate2+100)*outFilterMultimapNmax\n",
                    "id": "#star-alignreads.cwl/limitOutSAMoneReadBytes"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSJcollapsed"
                    },
                    "doc": "1000000\nint>0: max number of collapsed junctions\n",
                    "id": "#star-alignreads.cwl/limitOutSJcollapsed"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSJoneRead"
                    },
                    "doc": "1000\nint>0: max number of junctions for one read (including all multi-mappers)\n",
                    "id": "#star-alignreads.cwl/limitOutSJoneRead"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitSjdbInsertNsj"
                    },
                    "doc": "1000000\nint>=0: maximum number of junction to be inserted to the genome on the fly at\nthe mapping stage, including those from annotations and those detected in the\n1st step of the 2-pass run\n",
                    "id": "#star-alignreads.cwl/limitSjdbInsertNsj"
                },
                {
                    "type": "int",
                    "default": 10,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outBAMcompression"
                    },
                    "doc": "int: -1 to 10  BAM compression level, -1=default compression (6?), 0=no\ncompression, 10=maximum compression\n",
                    "id": "#star-alignreads.cwl/outBAMcompression"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outBAMsortingThreadN"
                    },
                    "doc": "int: >=0: number of threads for BAM sorting. 0 will default to\nmin(6,--runThreadN).\n",
                    "id": "#star-alignreads.cwl/outBAMsortingThreadN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFileNamePrefix",
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_name_prefix();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "string: output files name prefix (including full or relative path). Can\nonly be defined on the command line.\n",
                    "id": "#star-alignreads.cwl/outFileNamePrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterIntronMotifs"
                    },
                    "doc": "None\nstring: filter alignment using their motifs\nNone                           ... no filtering\nRemoveNoncanonical             ... filter out alignments that contain non-canonical junctions\nRemoveNoncanonicalUnannotated  ... filter out alignments that contain non-canonical unannotated junctions when using annotated splice junctions database. The annotated non-canonical junctions will be kept.\n",
                    "id": "#star-alignreads.cwl/outFilterIntronMotifs"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMatchNmin"
                    },
                    "doc": "0\nint: alignment will be output only if the number of matched bases is higher\nthan this value\n",
                    "id": "#star-alignreads.cwl/outFilterMatchNmin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMatchNminOverLread"
                    },
                    "doc": "0.66\nfloat: outFilterMatchNmin normalized to read length (sum of mates'' lengths\nfor paired-end reads)\n",
                    "id": "#star-alignreads.cwl/outFilterMatchNminOverLread"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNmax"
                    },
                    "doc": "10\nint: alignment will be output only if it has no more mismatches than this value\n",
                    "id": "#star-alignreads.cwl/outFilterMismatchNmax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNoverLmax"
                    },
                    "doc": "float: alignment will be output only if its ratio of mismatches to *mapped* length is less than or equal to this value.\n",
                    "id": "#star-alignreads.cwl/outFilterMismatchNoverLmax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNoverReadLmax"
                    },
                    "doc": "1\nfloat: alignment will be output only if its ratio of mismatches to *read* length\nis less than this value\n",
                    "id": "#star-alignreads.cwl/outFilterMismatchNoverReadLmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMultimapNmax"
                    },
                    "doc": "int: maximum number of loci the read is allowed to map to. Alignments (all of them) will be output only if\nthe read maps to no more loci than this value.\nOtherwise no alignments will be output, and the read will be counted as \"mapped to too many loci\"\nin the Log.final.out.\n",
                    "id": "#star-alignreads.cwl/outFilterMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMultimapScoreRange"
                    },
                    "doc": "1\nint: the score range below the maximum score for multimapping alignments\n",
                    "id": "#star-alignreads.cwl/outFilterMultimapScoreRange"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterScoreMin"
                    },
                    "doc": "0\nint: alignment will be output only if its score is higher than or equal to this value\n",
                    "id": "#star-alignreads.cwl/outFilterScoreMin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterScoreMinOverLread"
                    },
                    "doc": "0.66\nfloat: outFilterScoreMin normalized to read length (sum of mates'' lengths for\npaired-end reads)\n",
                    "id": "#star-alignreads.cwl/outFilterScoreMinOverLread"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterType"
                    },
                    "doc": "Normal\nstring: type of filtering\nNormal  ... standard filtering using only current alignment\nBySJout ... keep only those reads that contain junctions that passed filtering into SJ.out.tab\n",
                    "id": "#star-alignreads.cwl/outFilterType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outMultimapperOrder"
                    },
                    "doc": "Old_2.4\nstring: order of multimapping alignments in the output files\nOld_2.4             ... quasi-random order used before 2.5.0\nRandom              ... random order of alignments for each multi-mapper. Read mates (pairs) are always adjacent, all alignment for each read stay together. This option will become default in the future releases.\n",
                    "id": "#star-alignreads.cwl/outMultimapperOrder"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outQSconversionAdd"
                    },
                    "doc": "int: add this number to the quality score (e.g. to convert from Illumina to Sanger, use -31)\n",
                    "id": "#star-alignreads.cwl/outQSconversionAdd"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#star-alignreads.cwl/outReadsUnmapped/unmapped",
                            "symbols": [
                                "#star-alignreads.cwl/outReadsUnmapped/unmapped/None",
                                "#star-alignreads.cwl/outReadsUnmapped/unmapped/Fastx"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outReadsUnmapped"
                    },
                    "doc": "None\nstring: output of unmapped and partially mapped (i.e. mapped only one mate of a paired end read) reads in separate file(s).\nNone    ... no output\nFastx   ... output in separate fasta/fastq files, Unmapped.out.mate1/2\n",
                    "id": "#star-alignreads.cwl/outReadsUnmapped"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattrIHstart"
                    },
                    "doc": "1\nint>=0:                     start value for the IH attribute. 0 may be required\nby some downstream software, such as Cufflinks or StringTie.\n",
                    "id": "#star-alignreads.cwl/outSAMattrIHstart"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattrRGline"
                    },
                    "doc": "-\nstring(s): SAM/BAM read group line. The first word contains the read group identifier and must start with \"ID:\", e.g. --outSAMattrRGline ID:xxx CN:yy \"DS:z z z\".\nxxx will be added as RG tag to each output alignment. Any spaces in the tag values have to be double quoted.\nComma separated RG lines correspons to different (comma separated) input files in --readFilesIn. Commas have to be surrounded by spaces, e.g.\n--outSAMattrRGline ID:xxx , ID:zzz \"DS:z z\" , ID:yyy DS:yyyy\n",
                    "id": "#star-alignreads.cwl/outSAMattrRGline"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattributes"
                    },
                    "doc": "Standard\nstring: a string of desired SAM attributes, in the order desired for the output SAM\nNH HI AS nM NM MD jM jI XS ch ... any combination in any order\nStandard   ... NH HI AS nM\nAll        ... NH HI AS nM NM MD jM jI ch\nNone       ... no attributes\n",
                    "id": "#star-alignreads.cwl/outSAMattributes"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMfilter"
                    },
                    "doc": "None\nstring(s): filter the output into main SAM/BAM files\nKeepOnlyAddedReferences ... only keep the reads for which all alignments are to the extra reference sequences added with --genomeFastaFiles at the mapping stage.\nKeepAllAddedReferences ...  keep all alignments to the extra reference sequences added with --genomeFastaFiles at the mapping stage.\n",
                    "id": "#star-alignreads.cwl/outSAMfilter"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMflagAND"
                    },
                    "doc": "65535\nint: 0 to 65535: sam FLAG will be bitwise AND''d with this value, i.e. FLAG=FLAG\n& outSAMflagOR. This is applied after all flags have been set by STAR, but before\noutSAMflagOR. Can be used to unset specific bits that are not set otherwise.\n",
                    "id": "#star-alignreads.cwl/outSAMflagAND"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMflagOR"
                    },
                    "doc": "int: 0 to 65535: sam FLAG will be bitwise OR'd with this value, i.e.\nFLAG=FLAG | outSAMflagOR. This is applied after all flags have been set by\nSTAR, and after outSAMflagAND. Can be used to set specific bits that are\nnot set otherwise.\n",
                    "id": "#star-alignreads.cwl/outSAMflagOR"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderCommentFile"
                    },
                    "doc": "-\nstring: path to the file with @CO (comment) lines of the SAM header\n",
                    "id": "#star-alignreads.cwl/outSAMheaderCommentFile"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderHD"
                    },
                    "doc": "-\nstrings: @HD (header) line of the SAM header\n",
                    "id": "#star-alignreads.cwl/outSAMheaderHD"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderPG"
                    },
                    "doc": "-\nstrings: extra @PG (software) line of the SAM header (in addition to STAR)\n",
                    "id": "#star-alignreads.cwl/outSAMheaderPG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmapqUnique"
                    },
                    "doc": "255\nint: 0 to 255: the MAPQ value for unique mappers\n",
                    "id": "#star-alignreads.cwl/outSAMmapqUnique"
                },
                {
                    "type": "string",
                    "default": "Full",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmode"
                    },
                    "doc": "string: mode of SAM output\nNone ... no SAM output\nFull ... full SAM output\nNoQS ... full SAM but without quality scores\n",
                    "id": "#star-alignreads.cwl/outSAMmode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmultNmax"
                    },
                    "doc": "-1\nint: max number of multiple alignments for a read that will be output to the\nSAM/BAM files. -1 ... all alignments (up to --outFilterMultimapNmax) will be\noutput\n",
                    "id": "#star-alignreads.cwl/outSAMmultNmax"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMorder"
                    },
                    "doc": "Paired\nstring: type of sorting for the SAM output\nPaired: one mate after the other for all paired alignments\nPairedKeepInputOrder: one mate after the other for all paired alignments, the order is kept the same as in the input FASTQ files\n",
                    "id": "#star-alignreads.cwl/outSAMorder"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMprimaryFlag"
                    },
                    "doc": "OneBestScore\nstring: which alignments are considered primary - all others will be marked with 0x100 bit in the FLAG\nOneBestScore ... only one alignment with the best score is primary\nAllBestScore ... all alignments with the best score are primary\n",
                    "id": "#star-alignreads.cwl/outSAMprimaryFlag"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMreadID"
                    },
                    "doc": "Standard\nstring: read ID record type\nStandard ... first word (until space) from the FASTx read ID line, removing /1,/2 from the end\nNumber   ... read number (index) in the FASTx file\n",
                    "id": "#star-alignreads.cwl/outSAMreadID"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMstrandField"
                    },
                    "doc": "None\nstring: Cufflinks-like strand field flag\nNone        ... not used\nintronMotif ... strand derived from the intron motif. Reads with inconsistent and/or non-canonical introns are filtered out.\n",
                    "id": "#star-alignreads.cwl/outSAMstrandField"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "default": [
                        "BAM",
                        "Unsorted"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMtype"
                    },
                    "doc": "strings: type of SAM/BAM output\n1st word:\nBAM  ... output BAM without sorting\nSAM  ... output SAM without sorting\nNone ... no SAM/BAM output\n2nd, 3rd:\nUnsorted           ... standard unsorted\nSortedByCoordinate ... sorted by coordinate. This option will allocate extra memory for sorting which can be specified by --limitBAMsortRAM.\n",
                    "id": "#star-alignreads.cwl/outSAMtype"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMunmapped"
                    },
                    "doc": "string(s): output of unmapped reads in the SAM format\n  1st word:\n    None   ... no output\n    Within ... output unmapped reads within the main SAM file (i.e. Aligned.out.sam)\n  2nd word:\n    KeepPairs ... record unmapped mate for each alignment, and, in case of unsorted output, keep it adjacent to its mapped mate.\n    Only affects multi-mapping reads\n",
                    "id": "#star-alignreads.cwl/outSAMunmapped"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterCountTotalMin"
                    },
                    "doc": "3   1   1   1 \\n4 integers: minimum total (multi-mapping+unique) read count\nper junction for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3)\nGC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that\nmotif\\nJunctions are output if one of outSJfilterCountUniqueMin OR outSJfilterCountTotalMin\nconditions are satisfied\\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterCountTotalMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterCountUniqueMin"
                    },
                    "doc": "3   1   1   1 \\n4 integers: minimum uniquely mapping read count per junction\nfor: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC\nmotif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\\nJunctions\nare output if one of outSJfilterCountUniqueMin OR outSJfilterCountTotalMin\nconditions are satisfied\\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterCountUniqueMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterDistToOtherSJmin"
                    },
                    "doc": "10  0   5   10\n4 integers>=0: minimum allowed distance to other junctions' donor/acceptor\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterDistToOtherSJmin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterIntronMaxVsReadN"
                    },
                    "doc": "50000 100000 200000\nN integers>=0: maximum gap allowed for junctions supported by 1,2,3,,,N reads\ni.e. by default junctions supported by 1 read can have gaps <=50000b, by 2 reads: <=100000b, by 3 reads: <=200000. by >=4 reads any gap <=alignIntronMax\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterIntronMaxVsReadN"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterOverhangMin"
                    },
                    "doc": "30  12  12  12\n4 integers:    minimum overhang length for splice junctions on both sides for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterOverhangMin"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterReads"
                    },
                    "doc": "All\nstring: which reads to consider for collapsed splice junctions output\nAll: all reads, unique- and multi-mappers\nUnique: uniquely mapping reads only\n",
                    "id": "#star-alignreads.cwl/outSJfilterReads"
                },
                {
                    "type": "string",
                    "default": "Log",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outStd"
                    },
                    "doc": "Log\nstring: which output will be directed to stdout (standard out)\nLog                    ... log messages\nSAM                    ... alignments in SAM format (which normally are output to Aligned.out.sam file), normal standard output will go into Log.std.out\nBAM_Unsorted           ... alignments in BAM format, unsorted. Requires --outSAMtype BAM Unsorted\nBAM_SortedByCoordinate ... alignments in BAM format, unsorted. Requires --outSAMtype BAM SortedByCoordinate\nBAM_Quant              ... alignments to transcriptome in BAM format, unsorted. Requires --quantMode TranscriptomeSAM\n",
                    "id": "#star-alignreads.cwl/outStd"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outTmpDir"
                    },
                    "doc": "string: path to a directory that will be used as temporary by STAR. All contents of this directory will be removed!\n- the temp directory will default to outFileNamePrefix_STARtmp\n",
                    "id": "#star-alignreads.cwl/outTmpDir"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigNorm"
                    },
                    "doc": "RPM\nstring: type of normalization for the signal\nRPM    ... reads per million of mapped reads\nNone   ... no normalization, \"raw\" counts\n",
                    "id": "#star-alignreads.cwl/outWigNorm"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigReferencesPrefix"
                    },
                    "doc": "string: prefix matching reference names to include in the output wiggle\nfile, e.g. \"chr\", default \"-\" - include all references\n",
                    "id": "#star-alignreads.cwl/outWigReferencesPrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigStrand"
                    },
                    "doc": "Stranded\nstring: strandedness of wiggle/bedGraph output\nStranded   ...  separate strands, str1 and str2\nUnstranded ...  collapsed strands\n",
                    "id": "#star-alignreads.cwl/outWigStrand"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigType"
                    },
                    "doc": "None\nstring(s): type of signal output, e.g. \"bedGraph\" OR \"bedGraph read1_5p\". Requires sorted BAM: --outSAMtype BAM SortedByCoordinate .\n1st word:\nNone       ... no signal output\nbedGraph   ... bedGraph format\nwiggle     ... wiggle format\n2nd word:\nread1_5p   ... signal from only 5' of the 1st read, useful for CAGE/RAMPAGE etc\nread2      ... signal from only 2nd read\n",
                    "id": "#star-alignreads.cwl/outWigType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--parametersFiles"
                    },
                    "doc": "string: name of a user-defined parameters file, \"-\": none. Can only be\ndefined on the command line.\n",
                    "id": "#star-alignreads.cwl/parametersFiles"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantMode"
                    },
                    "doc": "-\nstring(s): types of quantification requested\n-                ... none\nTranscriptomeSAM ... output SAM/BAM alignments to transcriptome into a separate file\nGeneCounts       ... count reads per gene\n",
                    "id": "#star-alignreads.cwl/quantMode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantTranscriptomeBAMcompression"
                    },
                    "doc": "int: -1 to 10  transcriptome BAM compression level, -1=default compression\n(6?), 0=no compression, 10=maximum compression\n",
                    "id": "#star-alignreads.cwl/quantTranscriptomeBAMcompression"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantTranscriptomeBan"
                    },
                    "doc": "IndelSoftclipSingleend\nstring: prohibit various alignment type\nIndelSoftclipSingleend  ... prohibit indels, soft clipping and single-end alignments - compatible with RSEM\nSingleend               ... prohibit single-end alignments\n",
                    "id": "#star-alignreads.cwl/quantTranscriptomeBan"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readFilesCommand"
                    },
                    "doc": "string(s): command line to execute for each of the input file. This command should generate FASTA or FASTQ text and send it to stdout\nFor example: zcat - to uncompress .gz files, bzcat - to uncompress .bz2 files, etc.\n",
                    "id": "#star-alignreads.cwl/readFilesCommand"
                },
                {
                    "type": [
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readFilesIn"
                    },
                    "doc": "string(s): paths to files that contain input read1 (and, if needed,  read2)\n",
                    "id": "#star-alignreads.cwl/readFilesIn"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readMapNumber"
                    },
                    "doc": "-1\nint: number of reads to map from the beginning of the file\n-1: map all reads\n",
                    "id": "#star-alignreads.cwl/readMapNumber"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readMatesLengthsIn"
                    },
                    "doc": "string: Equal/NotEqual - lengths of names,sequences,qualities for both\nmates are the same  / not the same. NotEqual is safe in all situations.\n",
                    "id": "#star-alignreads.cwl/readMatesLengthsIn"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readNameSeparator"
                    },
                    "doc": "string(s): character(s) separating the part of the read names that will be\ntrimmed in output (read name after space is always trimmed)\n",
                    "id": "#star-alignreads.cwl/readNameSeparator"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runDirPerm"
                    },
                    "doc": "User_RWX\nstring: permissions for the directories created at the run-time.\nUser_RWX ... user-read/write/execute\nAll_RWX  ... all-read/write/execute (same as chmod 777)\n",
                    "id": "#star-alignreads.cwl/runDirPerm"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runRNGseed"
                    },
                    "doc": "777\nint: random number generator seed.\n",
                    "id": "#star-alignreads.cwl/runRNGseed"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreDelBase"
                    },
                    "doc": "-2\ndeletion extension penalty per base (in addition to scoreDelOpen)\n",
                    "id": "#star-alignreads.cwl/scoreDelBase"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreDelOpen"
                    },
                    "doc": "-2\ndeletion open penalty\n",
                    "id": "#star-alignreads.cwl/scoreDelOpen"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGap"
                    },
                    "doc": "0\nint: splice junction penalty (independent on intron motif)\n",
                    "id": "#star-alignreads.cwl/scoreGap"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapATAC"
                    },
                    "doc": "-8\nAT/AC  and GT/AT junction penalty  (in addition to scoreGap)\n",
                    "id": "#star-alignreads.cwl/scoreGapATAC"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapGCAG"
                    },
                    "doc": "-4\nGC/AG and CT/GC junction penalty (in addition to scoreGap)\n",
                    "id": "#star-alignreads.cwl/scoreGapGCAG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapNoncan"
                    },
                    "doc": "-8\nint: non-canonical junction penalty (in addition to scoreGap)\n",
                    "id": "#star-alignreads.cwl/scoreGapNoncan"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGenomicLengthLog2scale"
                    },
                    "doc": "-0.25\nextra score logarithmically scaled with genomic length of the alignment: scoreGenomicLengthLog2scale*log2(genomicLength)\n",
                    "id": "#star-alignreads.cwl/scoreGenomicLengthLog2scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreInsBase"
                    },
                    "doc": "-2\ninsertion extension penalty per base (in addition to scoreInsOpen)\n",
                    "id": "#star-alignreads.cwl/scoreInsBase"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreInsOpen"
                    },
                    "doc": "-2\ninsertion open penalty\n",
                    "id": "#star-alignreads.cwl/scoreInsOpen"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreStitchSJshift"
                    },
                    "doc": "1\nmaximum score reduction while searching for SJ boundaries inthe stitching step\n",
                    "id": "#star-alignreads.cwl/scoreStitchSJshift"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedMultimapNmax"
                    },
                    "doc": "10000\nint>0: only pieces that map fewer than this value are utilized in the stitching\nprocedure\n",
                    "id": "#star-alignreads.cwl/seedMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedNoneLociPerWindow"
                    },
                    "doc": "10\nnint>0: max number of one seed loci per window\n",
                    "id": "#star-alignreads.cwl/seedNoneLociPerWindow"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedPerReadNmax"
                    },
                    "doc": "1000\nint>0: max number of seeds per read\n",
                    "id": "#star-alignreads.cwl/seedPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedPerWindowNmax"
                    },
                    "doc": "50\nint>0: max number of seeds per window\n",
                    "id": "#star-alignreads.cwl/seedPerWindowNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchLmax"
                    },
                    "doc": "0\nint>=0: defines the maximum length of the seeds, if =0 max seed lengthis infinite\n",
                    "id": "#star-alignreads.cwl/seedSearchLmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchStartLmax"
                    },
                    "doc": "50\nint>0: defines the search start point through the read - the read is split into\npieces no longer than this value\n",
                    "id": "#star-alignreads.cwl/seedSearchStartLmax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchStartLmaxOverLread"
                    },
                    "doc": "1.0\nfloat: seedSearchStartLmax normalized to read length (sum of mates'' lengths\nfor paired-end reads)\n",
                    "id": "#star-alignreads.cwl/seedSearchStartLmaxOverLread"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbFileChrStartEnd"
                    },
                    "doc": "string(s): path to the files with genomic coordinates (chr <tab> start <tab>\nend <tab> strand) for the splice junction introns. Multiple files can be supplied\nwand will be concatenated.\n",
                    "id": "#star-alignreads.cwl/sjdbFileChrStartEnd"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFchrPrefix"
                    },
                    "doc": "string: prefix for chromosome names in a GTF file (e.g. 'chr' for using\nENSMEBL annotations with UCSC geneomes)\n",
                    "id": "#star-alignreads.cwl/sjdbGTFchrPrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFfeatureExon"
                    },
                    "doc": "exon\nstring: feature type in GTF file to be used as exons for building\ntranscripts\n",
                    "id": "#star-alignreads.cwl/sjdbGTFfeatureExon"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFfile"
                    },
                    "doc": "string: path to the GTF file with annotations\n",
                    "id": "#star-alignreads.cwl/sjdbGTFfile"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFtagExonParentGene"
                    },
                    "doc": "gene_id\nstring: tag name to be used as exons'' gene-parents (default \"gene_id\" works\nfor GTF files)\n",
                    "id": "#star-alignreads.cwl/sjdbGTFtagExonParentGene"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFtagExonParentTranscript"
                    },
                    "doc": "transcript_id\nstring: tag name to be used as exons' transcript-parents (default\n\"transcript_id\" works for GTF files)\n",
                    "id": "#star-alignreads.cwl/sjdbGTFtagExonParentTranscript"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbInsertSave"
                    },
                    "doc": "Basic\nstring: which files to save when sjdb junctions are inserted on the fly at the mapping step\nBasic ... only small junction / transcript files\nAll   ... all files including big Genome, SA and SAindex - this will create a complete genome directory\n",
                    "id": "#star-alignreads.cwl/sjdbInsertSave"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbOverhang"
                    },
                    "doc": "100\nint>0: length of the donor/acceptor sequence on each side of the junctions,\nideally = (mate_length - 1)\n",
                    "id": "#star-alignreads.cwl/sjdbOverhang"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbScore"
                    },
                    "doc": "2\nint: extra alignment score for alignmets that cross database junctions\n",
                    "id": "#star-alignreads.cwl/sjdbScore"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sysShell"
                    },
                    "doc": "string: path to the shell binary, preferrably bash, e.g. /bin/bash.\n- ... the default shell is executed, typically /bin/sh. This was reported to fail on some Ubuntu systems - then you need to specify path to bash.\n",
                    "id": "#star-alignreads.cwl/sysShell"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runThreadN"
                    },
                    "doc": "1\nint: number of threads to run STAR\n",
                    "id": "#star-alignreads.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--twopass1readsN"
                    },
                    "doc": "int: number of reads to process for the 1st step. Use very large number (or\ndefault -1) to map all reads in the first step.\n",
                    "id": "#star-alignreads.cwl/twopass1readsN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--twopassMode"
                    },
                    "doc": "None\nstring: 2-pass mapping mode.\nNone        ... 1-pass mapping\nBasic       ... basic 2-pass mapping, with all 1st pass junctions inserted into the genome indices on the fly\n",
                    "id": "#star-alignreads.cwl/twopassMode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winAnchorDistNbins"
                    },
                    "doc": "9\nint>0: max number of bins between two anchors that allows aggregation of anchors\ninto one window\n",
                    "id": "#star-alignreads.cwl/winAnchorDistNbins"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winAnchorMultimapNmax"
                    },
                    "doc": "50\nint>0: max number of loci anchors are allowed to map to\n",
                    "id": "#star-alignreads.cwl/winAnchorMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winBinNbits"
                    },
                    "doc": "16\nint>0: =log2(winBin), where winBin is the size of the bin for the windows/clustering,\neach window will occupy an integer number of bins.\n",
                    "id": "#star-alignreads.cwl/winBinNbits"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winFlankNbins"
                    },
                    "doc": "4\nint>0: log2(winFlank), where win Flank is the size of the left and right flanking\nregions for each window\n",
                    "id": "#star-alignreads.cwl/winFlankNbins"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winReadCoverageBasesMin"
                    },
                    "doc": "int>0: minimum number of bases covered by the seeds in a window , for STARlong algorithm only.\n",
                    "id": "#star-alignreads.cwl/winReadCoverageBasesMin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winReadCoverageRelativeMin"
                    },
                    "doc": "float>=0: minimum relative coverage of the read sequence by the seeds in a window, for STARlong algorithm only.\n",
                    "id": "#star-alignreads.cwl/winReadCoverageRelativeMin"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Aligned.out*"
                    },
                    "id": "#star-alignreads.cwl/aligned_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Log.final.out*"
                    },
                    "id": "#star-alignreads.cwl/log_final"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Log.out*"
                    },
                    "id": "#star-alignreads.cwl/log_out"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Log.progress.out*"
                    },
                    "id": "#star-alignreads.cwl/log_progress"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*SJ.out.tab*"
                    },
                    "id": "#star-alignreads.cwl/log_sj"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Log.std.out*"
                    },
                    "id": "#star-alignreads.cwl/log_std"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "*Log.final.out*",
                        "outputEval": "${\n  var uniquelyMappedReadsNumberRegex = /Uniquely mapped reads number.*/;\n  return parseInt(self[0].contents.match(uniquelyMappedReadsNumberRegex)[0].split(/\\|\\t/g)[1]);\n}\n"
                    },
                    "id": "#star-alignreads.cwl/uniquely_mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Unmapped.out.mate1*"
                    },
                    "id": "#star-alignreads.cwl/unmapped_mate_1_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Unmapped.out.mate2*"
                    },
                    "id": "#star-alignreads.cwl/unmapped_mate_2_file"
                }
            ],
            "baseCommand": [
                "STAR"
            ],
            "arguments": [
                "--runMode",
                "alignReads"
            ],
            "doc": "Tool runs STAR alignReads.\n\n`default_output_name_prefix` function returns output files prefix if `outFileNamePrefix` is not set. By default prefix\nis equal to basename of `readFilesIn`.\n",
            "id": "#star-alignreads.cwl",
            "http://schema.org/name": "star-alignreads",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/star-alignreads.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#star-metadata.yaml",
                "http://schema.org/name": "STAR",
                "http://schema.org/about": "Aligns RNA-seq reads to a reference genome using uncompressed suffix arrays.\nSTAR has a potential for accurately aligning long (several kilobases) reads that\nare emerging from the third-generation sequencing technologies\n",
                "http://schema.org/url": "https://github.com/alexdobin/STAR",
                "http://schema.org/codeRepository": "https://github.com/alexdobin/STAR",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "v2.5.3a",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#star-metadata.yaml/ 10.1093/bioinformatics/bts635",
                        "datePublished": "1 January 2013",
                        "http://schema.org/name": "STAR: ultrafast universal RNA-seq aligner",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Alexander Dobin"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Carrie A. Davis"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Felix Schlesinger"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Jorg Drenkow"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Chris Zaleski"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Sonali Jha"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Philippe Batut"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Mark Chaisson"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Thomas R. Gingeras1"
                            }
                        ],
                        "http://schema.org/url": "https://academic.oup.com/bioinformatics/article/29/1/15/272537"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/alexdobin/STAR/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Cold Spring Harbor Laboratory, Cold Spring Harbor, NY, USA",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Alexander Dobin",
                                "http://schema.org/description": "main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: STAR  [options]... --genomeDir REFERENCE   --readFilesIn R1.fq R2.fq\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { var basename = inputs.bedgraph_file.location.split('/').slice(-1)[0]; var root = basename.split('.').slice(0,-1).join('.'); var ext = \".bigWig\"; return (root == \"\")?basename+ext:root+ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/ucscuserapps:v358"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10
                    },
                    "doc": "Four column bedGraph file: <chrom> <start> <end> <value>\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/bedgraph_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "separate": false,
                        "position": 7,
                        "prefix": "-blockSize="
                    },
                    "doc": "Number of items to bundle in r-tree.  Default 256\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/block_size"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 11
                    },
                    "doc": "Two-column chromosome length file: <chromosome name> <size in bases>\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "separate": false,
                        "position": 6,
                        "prefix": "-itemsPerSlot="
                    },
                    "doc": "Number of data points bundled at lowest level. Default 1024\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/items_per_slot"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "If set, writes the output bigWig file to output_filename,\notherwise generates filename from default_output_filename()\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-unc"
                    },
                    "doc": "Disable compression\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/unc"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\"){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "id": "#ucsc-bedgraphtobigwig.cwl/bigwig_file"
                }
            ],
            "baseCommand": [
                "bedGraphToBigWig"
            ],
            "doc": "Tool converts bedGraph to bigWig file.\n\n`default_output_filename` function returns filename for generated bigWig if `output_filename` is not provided.\nDefault filename is generated on the base of `bedgraph_file` basename with the updated to `*.bigWig` extension.\n",
            "id": "#ucsc-bedgraphtobigwig.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#ucsc-metadata.yaml",
                "http://schema.org/name": "UCSC userApps",
                "http://schema.org/about": "UCSC genome browser bioinformatic utilities\n",
                "http://schema.org/url": "https://genome.ucsc.edu/util.html",
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "v358",
                    "http://schema.org/applicationCategory": "commandline tool",
                    "http://schema.org/downloadURL": "http://hgdownload.cse.ucsc.edu/admin/exe/userApps.v358.src.tgz"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ]
            },
            "http://schema.org/name": "ucsc-bedgraphtobigwig",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/ucsc-bedgraphtobigwig.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage:\n   bedGraphToBigWig in.bedGraph chrom.sizes out.bw\nwhere in.bedGraph is a four column file in the format:\n      <chrom> <start> <end> <value>\nand chrom.sizes is a two-column file/URL: <chromosome name> <size in bases>\nand out.bw is the output indexed big wig file.\nIf the assembly <db> is hosted by UCSC, chrom.sizes can be a URL like\n  http://hgdownload.cse.ucsc.edu/goldenPath/<db>/bigZips/<db>.chrom.sizes\nor you may use the script fetchChromSizes to download the chrom.sizes file.\nIf not hosted by UCSC, a chrom.sizes file can be generated by running\ntwoBitInfo on the assembly .2bit file.\nThe input bedGraph file must be sorted, use the unix sort command:\n  sort -k1,1 -k2,2n unsorted.bedGraph > sorted.bedGraph\noptions:\n   -blockSize=N - Number of items to bundle in r-tree.  Default 256\n   -itemsPerSlot=N - Number of data points bundled at lowest level. Default 1024\n   -unc - If set, do not use compression.\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { var ext = inputs.bam_file.basename.split('.').slice(-1)[0]; var root = inputs.bam_file.basename.split('.').slice(0,-1).join('.'); return inputs.output_filename?inputs.output_filename:root+\"_dedup.\"+ext; };"
                    ]
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": "${\n  return  [\n            {\n              \"entry\": inputs.bam_file,\n              \"entryname\": inputs.bam_file.basename,\n              \"writable\": true\n            }\n          ]\n}\n"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/umi_tools:1.0.1--py38h0213d0e_2"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-I"
                    },
                    "doc": "Input BAM file",
                    "id": "#umi-tools-dedup.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]; then\n  umi_tools dedup --random-seed=12345 \"${@:1}\"\nelse\n  echo \"Skip umi_tools dedup \" ${@:1}\nfi\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Bash function to run umi_tools dedup with all input parameters or skip it if trigger is false\n",
                    "id": "#umi-tools-dedup.cwl/bash_script"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--multimapping-detection-method=",
                        "separate": false
                    },
                    "doc": "Some aligners identify multimapping using bam tags.\nSetting this option to NH, X0 or XT will use these\ntags when selecting the best read amongst reads with\nthe same position and umi [default=none]\n",
                    "id": "#umi-tools-dedup.cwl/multimapping_detection_method"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "-S",
                        "valueFrom": "$(default_output_filename())"
                    },
                    "default": "",
                    "doc": "Output filename",
                    "id": "#umi-tools-dedup.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--output-stats=",
                        "separate": false
                    },
                    "default": "umi_tools_stats",
                    "doc": "Specify location to output stats",
                    "id": "#umi-tools-dedup.cwl/output_stats"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--paired"
                    },
                    "doc": "Inputs BAM file is paired end - output both read pairs.\nThis will also force the use of the template length to\ndetermine reads with the same mapping coordinates.\n",
                    "id": "#umi-tools-dedup.cwl/paired_end"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "inputBinding": {
                        "position": 6,
                        "valueFrom": "$(self?\"true\":\"false\")"
                    },
                    "doc": "If true - run umi_tools dedup, if false - return bam_file input, previously staged into the output directory\n",
                    "id": "#umi-tools-dedup.cwl/trigger"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${ return inputs.trigger?default_output_filename():inputs.bam_file.basename }\n"
                    },
                    "secondaryFiles": "${\n    if (inputs.bam_file.secondaryFiles && inputs.trigger == false){\n      return inputs.bam_file.secondaryFiles;\n    } else {\n      return \"null\";\n    }\n  }\n",
                    "id": "#umi-tools-dedup.cwl/dedup_bam_file"
                },
                {
                    "$import": "#umi-tools-dedup.cwl/output_stats"
                },
                {
                    "type": "File",
                    "id": "#umi-tools-dedup.cwl/stderr_log",
                    "outputBinding": {
                        "glob": "umi_tools_dedup_stderr.log"
                    }
                },
                {
                    "type": "File",
                    "id": "#umi-tools-dedup.cwl/stdout_log",
                    "outputBinding": {
                        "glob": "umi_tools_dedup_stdout.log"
                    }
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "stdout": "umi_tools_dedup_stdout.log",
            "stderr": "umi_tools_dedup_stderr.log",
            "doc": "Deduplicate BAM files based on the first mapping co-ordinate and the UMI attached to the read\nOnly -I, --paired and -S parameters are implemented.\n",
            "id": "#umi-tools-dedup.cwl",
            "http://schema.org/name": "umi-tools-dedup",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "UMI-Tools: Version 1.0.1\n\ndedup - Deduplicate reads using UMI and mapping coordinates\n\nUsage: umi_tools dedup [OPTIONS] [--stdin=IN_BAM] [--stdout=OUT_BAM]\n\n      note: If --stdout is ommited, standard out is output. To\n            generate a valid BAM file on standard out, please\n            redirect log with --log=LOGFILE or --log2stderr\n\nFor full UMI-tools documentation, see https://umi-tools.readthedocs.io/en/latest/\n\nOptions:\n  --version             show program's version number and exit\n\n  dedup-specific options:\n    --output-stats=STATS\n                        Specify location to output stats\n\n  Barcode extraction options:\n    --extract-umi-method=GET_UMI_METHOD\n                        how is the read UMI +/ cell barcode encoded?\n                        [default=read_id]\n    --umi-separator=UMI_SEP\n                        separator between read id and UMI\n    --umi-tag=UMI_TAG   tag containing umi\n    --umi-tag-split=UMI_TAG_SPLIT\n                        split UMI in tag and take the first element\n    --umi-tag-delimiter=UMI_TAG_DELIM\n                        concatenate UMI in tag separated by delimiter\n    --cell-tag=CELL_TAG\n                        tag containing cell barcode\n    --cell-tag-split=CELL_TAG_SPLIT\n                        split cell barcode in tag and take the first\n                        elementfor e.g 10X GEM tags\n    --cell-tag-delimiter=CELL_TAG_DELIM\n                        concatenate cell barcode in tag separated by delimiter\n\n  UMI grouping options:\n    --method=METHOD     method to use for umi grouping [default=directional]\n    --edit-distance-threshold=THRESHOLD\n                        Edit distance theshold at which to join two UMIs when\n                        grouping UMIs. [default=1]\n    --spliced-is-unique\n                        Treat a spliced read as different to an unspliced one\n                        [default=False]\n    --soft-clip-threshold=SOFT_CLIP_THRESHOLD\n                        number of bases clipped from 5' end before read is\n                        counted as spliced [default=4]\n    --read-length       use read length in addition to position and UMI to\n                        identify possible duplicates [default=False]\n\n  single-cell RNA-Seq options:\n    --per-gene          Group/Dedup/Count per gene. Must combine with either\n                        --gene-tag or --per-contig\n    --gene-tag=GENE_TAG\n                        Gene is defined by this bam tag [default=none]\n    --assigned-status-tag=ASSIGNED_TAG\n                        Bam tag describing whether read is assigned to a gene\n                        By defualt, this is set as the same tag as --gene-tag\n    --skip-tags-regex=SKIP_REGEX\n                        Used with --gene-tag. Ignore reads where the gene-tag\n                        matches this regex\n    --per-contig        group/dedup/count UMIs per contig (field 3 in BAM;\n                        RNAME), e.g for transcriptome where contig = gene\n    --gene-transcript-map=GENE_TRANSCRIPT_MAP\n                        File mapping transcripts to genes (tab separated)\n    --per-cell          group/dedup/count per cell\n\n  group/dedup options:\n    --buffer-whole-contig\n                        Read whole contig before outputting bundles:\n                        guarantees that no reads are missed, but increases\n                        memory usage\n    --multimapping-detection-method=DETECTION_METHOD\n                        Some aligners identify multimapping using bam tags.\n                        Setting this option to NH, X0 or XT will use these\n                        tags when selecting the best read amongst reads with\n                        the same position and umi [default=none]\n\n  SAM/BAM options:\n    --mapping-quality=MAPPING_QUALITY\n                        Minimum mapping quality for a read to be retained\n                        [default=0]\n    --unmapped-reads=UNMAPPED_READS\n                        How to handle unmapped reads. Options are 'discard',\n                        'use' or 'correct' [default=discard]\n    --chimeric-pairs=CHIMERIC_PAIRS\n                        How to handle chimeric read pairs. Options are\n                        'discard', 'use' or 'correct' [default=use]\n    --unpaired-reads=UNPAIRED_READS\n                        How to handle unpaired reads. Options are 'discard',\n                        'use' or 'correct' [default=use]\n    --ignore-umi        Ignore UMI and dedup only on position\n    --chrom=CHROM       Restrict to one chromosome\n    --subset=SUBSET     Use only a fraction of reads, specified by subset\n    -i, --in-sam        Input file is in sam format [default=False]\n    --paired            paired input BAM. [default=False]\n    -o, --out-sam       Output alignments in sam format [default=False]\n    --no-sort-output    Don't Sort the output\n\n  input/output options:\n    -I FILE, --stdin=FILE\n                        file to read stdin from [default = stdin].\n    -L FILE, --log=FILE\n                        file with logging information [default = stdout].\n    -E FILE, --error=FILE\n                        file with error information [default = stderr].\n    -S FILE, --stdout=FILE\n                        file where output is to go [default = stdout].\n    --temp-dir=FILE     Directory for temporary files. If not set, the bash\n                        environmental variable TMPDIR is used[default = None].\n    --log2stderr        send logging information to stderr [default = False].\n    --compresslevel=COMPRESSLEVEL\n                        Level of Gzip compression to use. Default (6)\n                        matchesGNU gzip rather than python gzip default (which\n                        is 9)\n\n  profiling options:\n    --timeit=TIMEIT_FILE\n                        store timeing information in file [none].\n    --timeit-name=TIMEIT_NAME\n                        name in timing file for this class of jobs [all].\n    --timeit-header     add header for timing information [none].\n\n  common options:\n    -v LOGLEVEL, --verbose=LOGLEVEL\n                        loglevel [1]. The higher, the more output.\n    -h, --help          output short help (command line options only).\n    --help-extended     Output full documentation\n    --random-seed=RANDOM_SEED\n                        random seed to initialize number generator with\n                        [none].\n"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_root = function(basename) { return basename.split('.').slice(0,1).join('.'); };"
                    ]
                }
            ],
            "sd:metadata": [
                "../metadata/rnaseq-header.cwl"
            ],
            "sd:upstream": {
                "genome_indices": "genome-indices.cwl"
            },
            "inputs": [
                {
                    "type": "File",
                    "label": "Annotation file",
                    "format": [
                        "http://edamontology.org/format_2306",
                        "http://edamontology.org/format_3475"
                    ],
                    "sd:upstreamSource": "genome_indices/annotation",
                    "doc": "GTF or TAB-separated annotation file",
                    "id": "#main/annotation_file"
                },
                {
                    "type": "File",
                    "label": "GTF annotation file",
                    "format": "http://edamontology.org/format_2306",
                    "sd:upstreamSource": "genome_indices/annotation_gtf",
                    "doc": "GTF annotation file",
                    "id": "#main/annotation_gtf_file"
                },
                {
                    "type": "Directory",
                    "label": "BowTie Ribosomal Indices",
                    "sd:upstreamSource": "genome_indices/ribosomal_indices",
                    "doc": "Path to Bowtie generated indices",
                    "id": "#main/bowtie_indices_folder"
                },
                {
                    "type": "File",
                    "label": "Chromosome length file",
                    "format": "http://edamontology.org/format_2330",
                    "sd:upstreamSource": "genome_indices/chrom_length",
                    "doc": "Chromosome length file",
                    "id": "#main/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 0,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Clip from 3p end",
                    "doc": "Number of bases to clip from the 3p end",
                    "id": "#main/clip_3p_end"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 0,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Clip from 5p end",
                    "doc": "Number of bases to clip from the 5p end",
                    "id": "#main/clip_5p_end"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Chromosome to be excluded in rpkm calculation",
                    "doc": "Chromosome to be excluded in rpkm calculation",
                    "id": "#main/exclude_chr"
                },
                {
                    "type": "File",
                    "label": "FASTQ input file",
                    "format": "http://edamontology.org/format_1930",
                    "doc": "Reads data in a FASTQ format",
                    "id": "#main/fastq_file"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "default": 1,
                    "label": "Minimum RPKM for Gene Body Average Tag Density Plot",
                    "doc": "Minimum RPKM for Gene Body Average Tag Density Plot",
                    "sd:layout": {
                        "advanced": true
                    },
                    "id": "#main/minimum_rpkm"
                },
                {
                    "type": "Directory",
                    "label": "STAR indices folder",
                    "sd:upstreamSource": "genome_indices/star_indices",
                    "doc": "Path to STAR generated indices",
                    "id": "#main/star_indices_folder"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 1,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Number of threads",
                    "doc": "Number of threads for those steps that support multi-threading",
                    "id": "#main/threads"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Use UMIs",
                    "doc": "Use UMIs (for FWD-UMI libraries)",
                    "id": "#main/use_umi"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "label": "BAM statistics report",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "BAM statistics report (right after alignment and sorting)",
                    "outputSource": "#main/get_bam_statistics/log_file",
                    "id": "#main/bam_statistics_report"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2572",
                    "label": "Coordinate sorted BAM alignment file (+index BAI)",
                    "doc": "Coordinate sorted BAM file and BAI index file",
                    "outputSource": "#main/samtools_sort_index_2/bam_bai_pair",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/bambai_pair/igvbrowser",
                                "optional": true,
                                "type": "alignment",
                                "format": "#main/bambai_pair/bam",
                                "name": "#main/bambai_pair/BAM Track",
                                "displayMode": "SQUISHED"
                            }
                        }
                    ],
                    "id": "#main/bambai_pair"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_3006",
                    "label": "BigWig file",
                    "doc": "Generated BigWig file",
                    "outputSource": "#main/bam_to_bigwig/bigwig_file",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/bigwig/igvbrowser",
                                "type": "wig",
                                "name": "#main/bigwig/BigWig Track",
                                "height": 120
                            }
                        }
                    ],
                    "id": "#main/bigwig"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "Bowtie alignment log",
                    "doc": "Bowtie alignment log file",
                    "outputSource": "#main/bowtie_aligner/log_file",
                    "id": "#main/bowtie_log"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "FASTQ statistics",
                    "doc": "fastx_quality_stats generated FASTQ file quality statistics file",
                    "outputSource": "#main/fastx_quality_stats/statistics_file",
                    "sd:visualPlugins": [
                        {
                            "line": {
                                "tab": "QC Plots",
                                "Title": "Base frequency plot",
                                "xAxisTitle": "Nucleotide position",
                                "yAxisTitle": "Frequency",
                                "colors": [
                                    "#b3de69",
                                    "#888888",
                                    "#fb8072",
                                    "#fdc381",
                                    "#99c0db"
                                ],
                                "data": [
                                    "$13",
                                    "$14",
                                    "$15",
                                    "$16",
                                    "$17"
                                ]
                            }
                        },
                        {
                            "boxplot": {
                                "tab": "QC Plots",
                                "Title": "Quality Control",
                                "xAxisTitle": "Nucleotide position",
                                "yAxisTitle": "Quality score",
                                "colors": [
                                    "#b3de69",
                                    "#888888",
                                    "#fb8072",
                                    "#fdc381",
                                    "#99c0db"
                                ],
                                "data": [
                                    "$11",
                                    "$7",
                                    "$8",
                                    "$9",
                                    "$12"
                                ]
                            }
                        }
                    ],
                    "id": "#main/fastx_statistics"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_3508",
                    "label": "Gene body average tag density plot for all isoforms longer than 1000 bp",
                    "doc": "Gene body average tag density plot for all isoforms longer than 1000 bp in PDF format",
                    "outputSource": "#main/get_gene_body/gene_body_plot_pdf",
                    "id": "#main/gene_body_plot_pdf"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_3475",
                    "label": "Gene body average tag density plot for all isoforms longer than 1000 bp",
                    "doc": "Gene body average tag density plot for all isoforms longer than 1000 bp in TSV format",
                    "outputSource": "#main/get_gene_body/gene_body_report_file",
                    "sd:visualPlugins": [
                        {
                            "line": {
                                "tab": "QC Plots",
                                "Title": "Gene body average tag density plot",
                                "xAxisTitle": "Gene body percentile (5' -> 3')",
                                "yAxisTitle": "Average Tag Density (per percentile)",
                                "colors": [
                                    "#232C15"
                                ],
                                "data": [
                                    "$2"
                                ],
                                "comparable": "gbatdp"
                            }
                        }
                    ],
                    "id": "#main/gene_body_report"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Bowtie, STAR and GEEP mapping stats",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "Processed and combined Bowtie & STAR aligner and GEEP logs",
                    "outputSource": "#main/get_stat/collected_statistics_tsv",
                    "sd:visualPlugins": [
                        {
                            "tableView": {
                                "vertical": true,
                                "tab": "Overview"
                            }
                        }
                    ],
                    "sd:preview": {
                        "sd:visualPlugins": [
                            {
                                "pie": {
                                    "colors": [
                                        "#b3de69",
                                        "#99c0db",
                                        "#fdc381",
                                        "#fb8072",
                                        "#778899"
                                    ],
                                    "data": [
                                        "$2",
                                        "$3",
                                        "$4",
                                        "$5",
                                        "$6"
                                    ]
                                }
                            }
                        ]
                    },
                    "id": "#main/get_formatted_stats"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "YAML formatted combined log",
                    "format": "http://edamontology.org/format_3750",
                    "doc": "YAML formatted combined log",
                    "outputSource": "#main/get_stat/collected_statistics_yaml",
                    "id": "#main/get_stat_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Markdown formatted combined log",
                    "format": "http://edamontology.org/format_3835",
                    "doc": "Markdown formatted combined log",
                    "outputSource": "#main/get_stat/collected_statistics_md",
                    "sd:visualPlugins": [
                        {
                            "markdownView": {
                                "tab": "Overview"
                            }
                        }
                    ],
                    "id": "#main/get_stat_markdown"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_3475",
                    "label": "HTSeq: read counts grouped by gene_id",
                    "doc": "Feature counts from htseq-count grouped by gene_id",
                    "outputSource": "#main/htseq_count_gene_expression/feature_counts_report_file",
                    "id": "#main/htseq_count_report_file"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "HTSeq: stderr log",
                    "doc": "HTSeq: stderr log",
                    "outputSource": "#main/htseq_count_gene_expression/stderr_log",
                    "id": "#main/htseq_count_stderr_log"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "HTSeq: stdout log",
                    "doc": "HTSeq: stdout log",
                    "outputSource": "#main/htseq_count_gene_expression/stdout_log",
                    "id": "#main/htseq_count_stdout_log"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_3475",
                    "label": "raw reads grouped by common TSS",
                    "doc": "raw reads grouped by common TSS",
                    "outputSource": "#main/group_isoforms/common_tss_file",
                    "id": "#main/rpkm_common_tss"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_3508",
                    "label": "RPKM distribution plot for isoforms",
                    "doc": "RPKM distribution plot for isoforms in PDF format",
                    "outputSource": "#main/get_gene_body/rpkm_distribution_plot_pdf",
                    "id": "#main/rpkm_distribution_plot_pdf"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_3475",
                    "label": "raw reads grouped by gene name",
                    "doc": "raw reads grouped by gene name",
                    "outputSource": "#main/group_isoforms/genes_file",
                    "sd:visualPlugins": [
                        {
                            "syncfusiongrid": {
                                "tab": "Gene Expression",
                                "Title": "raw reads grouped by gene name"
                            }
                        }
                    ],
                    "id": "#main/rpkm_genes"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR final log",
                    "doc": "STAR Log.final.out",
                    "outputSource": "#main/star_aligner/log_final",
                    "id": "#main/star_final_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR log out",
                    "doc": "STAR Log.out",
                    "outputSource": "#main/star_aligner/log_out",
                    "id": "#main/star_out_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR progress log",
                    "doc": "STAR Log.progress.out",
                    "outputSource": "#main/star_aligner/log_progress",
                    "id": "#main/star_progress_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR sj log",
                    "doc": "STAR SJ.out.tab",
                    "outputSource": "#main/star_aligner/log_sj",
                    "id": "#main/star_sj_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR stdout log",
                    "doc": "STAR Log.std.out",
                    "outputSource": "#main/star_aligner/log_std",
                    "id": "#main/star_stdout_log"
                },
                {
                    "type": "File",
                    "label": "cutadapt report",
                    "doc": "cutadapt generated log",
                    "outputSource": "#main/umisep_cutadapt/report_file",
                    "id": "#main/trim_report"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "label": "umi_tools dedup stats",
                    "doc": "umi_tools dedup stats",
                    "outputSource": "#main/umi_tools_dedup/output_stats",
                    "id": "#main/umi_tools_dedup_stats"
                },
                {
                    "type": "File",
                    "label": "umi_tools dedup stderr log",
                    "doc": "umi_tools dedup stderr log",
                    "outputSource": "#main/umi_tools_dedup/stderr_log",
                    "id": "#main/umi_tools_dedup_stderr"
                },
                {
                    "type": "File",
                    "label": "umi_tools dedup stdout log",
                    "doc": "umi_tools dedup stdout log",
                    "outputSource": "#main/umi_tools_dedup/stdout_log",
                    "id": "#main/umi_tools_dedup_stdout"
                }
            ],
            "steps": [
                {
                    "run": "#bam-bedgraph-bigwig.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index_2/bam_bai_pair",
                            "id": "#main/bam_to_bigwig/bam_file"
                        },
                        {
                            "source": "#main/chrom_length_file",
                            "id": "#main/bam_to_bigwig/chrom_length_file"
                        },
                        {
                            "source": "#main/star_aligner/uniquely_mapped_reads_number",
                            "id": "#main/bam_to_bigwig/mapped_reads_number"
                        }
                    ],
                    "out": [
                        "#main/bam_to_bigwig/bigwig_file"
                    ],
                    "id": "#main/bam_to_bigwig"
                },
                {
                    "run": "#bowtie-alignreads.cwl",
                    "in": [
                        {
                            "default": true,
                            "id": "#main/bowtie_aligner/best"
                        },
                        {
                            "source": "#main/clip_3p_end",
                            "id": "#main/bowtie_aligner/clip_3p_end"
                        },
                        {
                            "source": "#main/clip_5p_end",
                            "id": "#main/bowtie_aligner/clip_5p_end"
                        },
                        {
                            "source": "#main/bowtie_indices_folder",
                            "id": "#main/bowtie_aligner/indices_folder"
                        },
                        {
                            "default": 1,
                            "id": "#main/bowtie_aligner/m"
                        },
                        {
                            "default": true,
                            "id": "#main/bowtie_aligner/sam"
                        },
                        {
                            "default": true,
                            "id": "#main/bowtie_aligner/strata"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/bowtie_aligner/threads"
                        },
                        {
                            "source": "#main/rename/target_file",
                            "id": "#main/bowtie_aligner/upstream_filelist"
                        },
                        {
                            "default": 3,
                            "id": "#main/bowtie_aligner/v"
                        }
                    ],
                    "out": [
                        "#main/bowtie_aligner/log_file"
                    ],
                    "id": "#main/bowtie_aligner"
                },
                {
                    "run": "#extract-fastq.cwl",
                    "in": [
                        {
                            "source": "#main/fastq_file",
                            "id": "#main/extract_fastq/compressed_file"
                        }
                    ],
                    "out": [
                        "#main/extract_fastq/fastq_file"
                    ],
                    "id": "#main/extract_fastq"
                },
                {
                    "run": "#fastx-quality-stats.cwl",
                    "in": [
                        {
                            "source": "#main/rename/target_file",
                            "id": "#main/fastx_quality_stats/input_file"
                        }
                    ],
                    "out": [
                        "#main/fastx_quality_stats/statistics_file"
                    ],
                    "id": "#main/fastx_quality_stats"
                },
                {
                    "run": "#samtools-stats.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index_2/bam_bai_pair",
                            "id": "#main/get_bam_statistics/bambai_pair"
                        },
                        {
                            "source": "#main/samtools_sort_index_2/bam_bai_pair",
                            "valueFrom": "$(get_root(self.basename)+\"_bam_statistics_report.txt\")",
                            "id": "#main/get_bam_statistics/output_filename"
                        }
                    ],
                    "out": [
                        "#main/get_bam_statistics/log_file"
                    ],
                    "id": "#main/get_bam_statistics"
                },
                {
                    "run": "#plugin-plot-rna.cwl",
                    "in": [
                        {
                            "source": "#main/annotation_file",
                            "id": "#main/get_gene_body/annotation_file"
                        },
                        {
                            "source": "#main/samtools_sort_index_2/bam_bai_pair",
                            "id": "#main/get_gene_body/bambai_pair"
                        },
                        {
                            "source": "#main/rpkm_calculation/isoforms_file",
                            "id": "#main/get_gene_body/isoforms_file"
                        },
                        {
                            "source": "#main/star_aligner/uniquely_mapped_reads_number",
                            "id": "#main/get_gene_body/mapped_reads_number"
                        },
                        {
                            "source": "#main/minimum_rpkm",
                            "id": "#main/get_gene_body/minimum_rpkm"
                        },
                        {
                            "default": "no",
                            "id": "#main/get_gene_body/strand_specificity"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/get_gene_body/threads"
                        }
                    ],
                    "out": [
                        "#main/get_gene_body/gene_body_report_file",
                        "#main/get_gene_body/gene_body_plot_pdf",
                        "#main/get_gene_body/rpkm_distribution_plot_pdf"
                    ],
                    "id": "#main/get_gene_body"
                },
                {
                    "run": "#collect-statistics-rna-quantseq.cwl",
                    "in": [
                        {
                            "source": "#main/get_bam_statistics/log_file",
                            "id": "#main/get_stat/bam_statistics_report"
                        },
                        {
                            "source": "#main/bowtie_aligner/log_file",
                            "id": "#main/get_stat/bowtie_alignment_report"
                        },
                        {
                            "source": "#main/rpkm_calculation/isoforms_file",
                            "id": "#main/get_stat/isoforms_file"
                        },
                        {
                            "source": "#main/star_aligner/log_final",
                            "id": "#main/get_stat/star_alignment_report"
                        }
                    ],
                    "out": [
                        "#main/get_stat/collected_statistics_yaml",
                        "#main/get_stat/collected_statistics_tsv",
                        "#main/get_stat/collected_statistics_md"
                    ],
                    "id": "#main/get_stat"
                },
                {
                    "in": [
                        {
                            "source": "#main/rpkm_calculation/isoforms_file",
                            "id": "#main/group_isoforms/isoforms_file"
                        }
                    ],
                    "out": [
                        "#main/group_isoforms/genes_file",
                        "#main/group_isoforms/common_tss_file",
                        "#main/group_isoforms/error_file"
                    ],
                    "run": {
                        "cwlVersion": "v1.0",
                        "class": "CommandLineTool",
                        "hints": [
                            {
                                "class": "DockerRequirement",
                                "dockerPull": "biowardrobe2/scidap-deseq:v0.0.20"
                            }
                        ],
                        "inputs": [
                            {
                                "type": [
                                    "null",
                                    "string"
                                ],
                                "default": "#!/bin/bash\n\nFILE=$0\nBASENAME=$(basename \"$FILE\")\n\nget_gene_n_tss.R --isoforms \"${FILE}\" --gene grouped.genes.tsv --tss grouped.common_tss.tsv\n\nsed -ibak 's/[[:space:]]\\{1,\\}[^[:space:]]\\{1,\\}$//' grouped.genes.tsv\nsed -ibak 's/[[:space:]]\\{1,\\}[^[:space:]]\\{1,\\}$//' grouped.common_tss.tsv\nrm -f ./*bak\n",
                                "inputBinding": {
                                    "position": 1
                                },
                                "doc": "Bash function to run awk & cutadapt from Lexogen with all input parameters or skip it if trigger is false\n",
                                "id": "#main/group_isoforms/13657a59-3fc8-484c-8c18-467f5ef61b41/bash_script"
                            },
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 5
                                },
                                "id": "#main/group_isoforms/13657a59-3fc8-484c-8c18-467f5ef61b41/isoforms_file"
                            }
                        ],
                        "outputs": [
                            {
                                "type": "File",
                                "outputBinding": {
                                    "glob": "$(inputs.common_tss_file?inputs.common_tss_file:\"*common_tss.tsv\")"
                                },
                                "doc": "Output TSV common tss expression file",
                                "id": "#main/group_isoforms/13657a59-3fc8-484c-8c18-467f5ef61b41/common_tss_file"
                            },
                            {
                                "type": "File",
                                "id": "#main/group_isoforms/13657a59-3fc8-484c-8c18-467f5ef61b41/error_file",
                                "outputBinding": {
                                    "glob": "group_isoforms_error.log"
                                }
                            },
                            {
                                "type": "File",
                                "outputBinding": {
                                    "glob": "$(inputs.genes_filename?inputs.genes_filename:\"*genes.tsv\")"
                                },
                                "doc": "Output TSV gene expression file",
                                "id": "#main/group_isoforms/13657a59-3fc8-484c-8c18-467f5ef61b41/genes_file"
                            }
                        ],
                        "baseCommand": [
                            "bash",
                            "-c"
                        ],
                        "stderr": "group_isoforms_error.log",
                        "id": "#main/group_isoforms/13657a59-3fc8-484c-8c18-467f5ef61b41"
                    },
                    "id": "#main/group_isoforms"
                },
                {
                    "run": "#htseq-count.cwl",
                    "in": [
                        {
                            "default": "transcript_id",
                            "id": "#main/htseq_count_gene_expression/additional_id"
                        },
                        {
                            "source": "#main/samtools_sort_index_2/bam_bai_pair",
                            "id": "#main/htseq_count_gene_expression/alignment_bam_file"
                        },
                        {
                            "source": "#main/annotation_gtf_file",
                            "id": "#main/htseq_count_gene_expression/annotation_gtf_file"
                        },
                        {
                            "default": "gene_id",
                            "id": "#main/htseq_count_gene_expression/feature_id"
                        },
                        {
                            "default": "exon",
                            "id": "#main/htseq_count_gene_expression/feature_type"
                        },
                        {
                            "default": "yes",
                            "id": "#main/htseq_count_gene_expression/strand_specific"
                        }
                    ],
                    "out": [
                        "#main/htseq_count_gene_expression/feature_counts_report_file",
                        "#main/htseq_count_gene_expression/stdout_log",
                        "#main/htseq_count_gene_expression/stderr_log"
                    ],
                    "id": "#main/htseq_count_gene_expression"
                },
                {
                    "run": "#rename.cwl",
                    "in": [
                        {
                            "source": "#main/umisep_cutadapt/trimmed_file",
                            "id": "#main/rename/source_file"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "valueFrom": "$(self.basename)",
                            "id": "#main/rename/target_filename"
                        }
                    ],
                    "out": [
                        "#main/rename/target_file"
                    ],
                    "id": "#main/rename"
                },
                {
                    "run": "#geep.cwl",
                    "in": [
                        {
                            "source": "#main/annotation_file",
                            "id": "#main/rpkm_calculation/annotation_file"
                        },
                        {
                            "source": "#main/samtools_sort_index_2/bam_bai_pair",
                            "id": "#main/rpkm_calculation/bam_file"
                        },
                        {
                            "source": "#main/exclude_chr",
                            "id": "#main/rpkm_calculation/exclude_chr"
                        },
                        {
                            "default": 0,
                            "id": "#main/rpkm_calculation/max_cycles"
                        },
                        {
                            "default": 0,
                            "id": "#main/rpkm_calculation/rpkm_threshold"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/rpkm_calculation/threads"
                        }
                    ],
                    "out": [
                        "#main/rpkm_calculation/isoforms_file"
                    ],
                    "id": "#main/rpkm_calculation"
                },
                {
                    "run": "#samtools-sort-index.cwl",
                    "in": [
                        {
                            "source": "#main/star_aligner/aligned_file",
                            "id": "#main/samtools_sort_index_1/sort_input"
                        },
                        {
                            "source": "#main/rename/target_file",
                            "valueFrom": "$(self.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+'.bam')",
                            "id": "#main/samtools_sort_index_1/sort_output_filename"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/samtools_sort_index_1/threads"
                        }
                    ],
                    "out": [
                        "#main/samtools_sort_index_1/bam_bai_pair"
                    ],
                    "id": "#main/samtools_sort_index_1"
                },
                {
                    "run": "#samtools-sort-index.cwl",
                    "in": [
                        {
                            "source": "#main/umi_tools_dedup/dedup_bam_file",
                            "id": "#main/samtools_sort_index_2/sort_input"
                        },
                        {
                            "source": "#main/rename/target_file",
                            "valueFrom": "$(self.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+'.bam')",
                            "id": "#main/samtools_sort_index_2/sort_output_filename"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/samtools_sort_index_2/threads"
                        }
                    ],
                    "out": [
                        "#main/samtools_sort_index_2/bam_bai_pair"
                    ],
                    "id": "#main/samtools_sort_index_2"
                },
                {
                    "run": "#star-alignreads.cwl",
                    "in": [
                        {
                            "default": 1,
                            "id": "#main/star_aligner/alignSJDBoverhangMin"
                        },
                        {
                            "source": "#main/clip_3p_end",
                            "id": "#main/star_aligner/clip3pNbases"
                        },
                        {
                            "source": "#main/clip_5p_end",
                            "id": "#main/star_aligner/clip5pNbases"
                        },
                        {
                            "source": "#main/star_indices_folder",
                            "id": "#main/star_aligner/genomeDir"
                        },
                        {
                            "default": 5,
                            "id": "#main/star_aligner/outFilterMismatchNmax"
                        },
                        {
                            "default": 1,
                            "id": "#main/star_aligner/outFilterMultimapNmax"
                        },
                        {
                            "source": "#main/rename/target_file",
                            "id": "#main/star_aligner/readFilesIn"
                        },
                        {
                            "default": 15,
                            "id": "#main/star_aligner/seedSearchStartLmax"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/star_aligner/threads"
                        }
                    ],
                    "out": [
                        "#main/star_aligner/aligned_file",
                        "#main/star_aligner/log_final",
                        "#main/star_aligner/uniquely_mapped_reads_number",
                        "#main/star_aligner/log_out",
                        "#main/star_aligner/log_progress",
                        "#main/star_aligner/log_std",
                        "#main/star_aligner/log_sj"
                    ],
                    "id": "#main/star_aligner"
                },
                {
                    "run": "#umi-tools-dedup.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index_1/bam_bai_pair",
                            "id": "#main/umi_tools_dedup/bam_file"
                        },
                        {
                            "default": "NH",
                            "id": "#main/umi_tools_dedup/multimapping_detection_method"
                        },
                        {
                            "source": "#main/use_umi",
                            "id": "#main/umi_tools_dedup/trigger"
                        }
                    ],
                    "out": [
                        "#main/umi_tools_dedup/dedup_bam_file",
                        "#main/umi_tools_dedup/stdout_log",
                        "#main/umi_tools_dedup/stderr_log",
                        "#main/umi_tools_dedup/output_stats"
                    ],
                    "id": "#main/umi_tools_dedup"
                },
                {
                    "in": [
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "id": "#main/umisep_cutadapt/input_file"
                        },
                        {
                            "source": "#main/use_umi",
                            "id": "#main/umisep_cutadapt/trigger"
                        }
                    ],
                    "out": [
                        "#main/umisep_cutadapt/trimmed_file",
                        "#main/umisep_cutadapt/report_file"
                    ],
                    "run": {
                        "cwlVersion": "v1.0",
                        "class": "CommandLineTool",
                        "hints": [
                            {
                                "class": "DockerRequirement",
                                "dockerPull": "scidap/trimgalore:v0.6.6"
                            }
                        ],
                        "inputs": [
                            {
                                "type": [
                                    "null",
                                    "string"
                                ],
                                "default": "#!/bin/bash\nFILE=$1\nBASENAME=$(basename \"$FILE\")\nif [ \"$0\" = \"true\" ]; then\n  cat ${FILE} | awk '\n  NR%4==1{ rd_name=$1; rd_info=$2 }\n  NR%4==2{ umi=substr($1,1,10); rd_seq=substr($1,11) }\n  NR%4==0{ print rd_name\"_\"umi\" \"rd_info; print rd_seq; print \"+\"; print substr($1,11) }' |\n  cutadapt -m 20 -O 20 -a \"polyA=A{20}\" -a \"QUALITY=G{20}\" -n 2 - |\n  cutadapt  -m 20 -O 3 --nextseq-trim=10  -a \"r1adapter=A{18}AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC;min_overlap=3;max_error_rate=0.100000\" - |\n  cutadapt -m 20 -O 3 -a \"r1polyA=A{18}\" - |\n  cutadapt -m 20 -O 20 -g \"r1adapter=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC;min_overlap=20\" --discard-trimmed -o trimmed_${BASENAME} -\nelse\n  cp ${FILE} trimmed_${BASENAME}\nfi\n",
                                "inputBinding": {
                                    "position": 1
                                },
                                "doc": "Bash function to run awk & cutadapt from Lexogen with all input parameters or skip it if trigger is false\n",
                                "id": "#main/umisep_cutadapt/1c5799ed-fd48-4f5e-9b1d-1906ce6d4ed0/bash_script"
                            },
                            {
                                "type": [
                                    "File"
                                ],
                                "inputBinding": {
                                    "position": 3
                                },
                                "doc": "Input FASTQ file\n",
                                "id": "#main/umisep_cutadapt/1c5799ed-fd48-4f5e-9b1d-1906ce6d4ed0/input_file"
                            },
                            {
                                "type": [
                                    "null",
                                    "boolean"
                                ],
                                "default": true,
                                "inputBinding": {
                                    "position": 2,
                                    "valueFrom": "$(self?\"true\":\"false\")"
                                },
                                "id": "#main/umisep_cutadapt/1c5799ed-fd48-4f5e-9b1d-1906ce6d4ed0/trigger"
                            }
                        ],
                        "outputs": [
                            {
                                "type": "File",
                                "id": "#main/umisep_cutadapt/1c5799ed-fd48-4f5e-9b1d-1906ce6d4ed0/report_file",
                                "outputBinding": {
                                    "glob": "umisep_cutadapt.log"
                                }
                            },
                            {
                                "type": "File",
                                "outputBinding": {
                                    "glob": "trimmed_*"
                                },
                                "id": "#main/umisep_cutadapt/1c5799ed-fd48-4f5e-9b1d-1906ce6d4ed0/trimmed_file"
                            }
                        ],
                        "baseCommand": [
                            "bash",
                            "-c"
                        ],
                        "stderr": "umisep_cutadapt.log",
                        "id": "#main/umisep_cutadapt/1c5799ed-fd48-4f5e-9b1d-1906ce6d4ed0"
                    },
                    "id": "#main/umisep_cutadapt"
                }
            ],
            "label": "QuantSeq 3' mRNA-Seq single-read",
            "doc": "### Pipeline for Lexogen's QuantSeq 3' mRNA-Seq Library Prep Kit FWD for Illumina\n\n[Lexogen original documentation](https://www.lexogen.com/quantseq-3mrna-sequencing/)\n\n* Cost-saving and streamlined globin mRNA depletion during QuantSeq library preparation\n* Genome-wide analysis of gene expression\n* Cost-efficient alternative to microarrays and standard RNA-Seq\n* Down to 100 pg total RNA input\n* Applicable for low quality and FFPE samples\n* Single-read sequencing of up to 9,216 samples/lane\n* Dual indexing and Unique Molecular Identifiers (UMIs) are available\n\n### QuantSeq 3\u2019 mRNA-Seq Library Prep Kit FWD for Illumina\n\nThe QuantSeq FWD Kit is a library preparation protocol designed to generate Illumina compatible libraries of sequences close to the 3\u2019 end of polyadenylated RNA.\n\nQuantSeq FWD contains the Illumina Read 1 linker sequence in the second strand synthesis primer,\nhence NGS reads are generated towards the poly(A) tail, directly reflecting the mRNA sequence (see workflow).\nThis version is the recommended standard for gene expression analysis.\nLexogen furthermore provides a high-throughput version with optional dual indexing (i5 and i7 indices) allowing up to 9,216 samples to be multiplexed in one lane.\n\n#### Analysis of Low Input and Low Quality Samples\n\nThe required input amount of total RNA is as low as 100 pg.\nQuantSeq is suitable to reproducibly generate libraries from low quality RNA, including FFPE samples.\nSee Fig.1 and 2 for a comparison of two different RNA qualities (FFPE and fresh frozen cryo-block) of the same sample.\n\n![Fig 1](https://www.lexogen.com/wp-content/uploads/2017/02/Correlation_Samples.jpg)\n\nFigure 1 | Correlation of gene counts of FFPE and cryo samples.\n\n![Fig 2](https://www.lexogen.com/wp-content/uploads/2017/02/Venn_diagrams.jpg)\n\nFigure 2 | Venn diagrams of genes detected by QuantSeq at a uniform read depth of 2.5 M reads in FFPE and cryo samples with 1, 5, and 10 reads/gene thresholds.\n\n#### Mapping of Transcript End Sites\n\nBy using longer reads QuantSeq FWD allows to exactly pinpoint the 3\u2019 end of poly(A) RNA (see Fig. 3) and therefore obtain accurate information about the 3\u2019 UTR.\n\n![Figure 3](https://www.lexogen.com/wp-content/uploads/2017/02/Read_Coverage.jpg)\n\nFigure 3 | QuantSeq read coverage versus normalized transcript length of NGS libraries derived from FFPE-RNA (blue) and cryo-preserved RNA (red).\n\n\n### Current workflow should be used only with the single-end RNA-Seq data. It performs the following steps:\n\n1. Separates UMIes and trims adapters from input FASTQ file\n2. Uses ```STAR``` to align reads from input FASTQ file according to the predefined reference indices; generates unsorted BAM file and alignment statistics file\n3. Uses ```fastx_quality_stats``` to analyze input FASTQ file and generates quality statistics file\n4. Uses ```samtools sort``` and generates coordinate sorted BAM(+BAI) file pair from the unsorted BAM file obtained on the step 2 (after running STAR)\n5. Uses ```umi_tools dedup``` and generates final filtered sorted BAM(+BAI) file pair\n6. Generates BigWig file on the base of sorted BAM file\n7. Maps input FASTQ file to predefined rRNA reference indices using ```bowtie``` to define the level of rRNA contamination; exports resulted statistics to file\n8. Calculates isoform expression level for the sorted BAM file and GTF/TAB annotation file using GEEP reads-counting utility; exports results to file",
            "id": "#main",
            "http://schema.org/name": "QuantSeq 3' mRNA-Seq single-read",
            "http://schema.org/alternateName": "Run QuantSeq 3' mRNA-Seq basic analysis with single-end data file",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/datirium/workflows/master/workflows/trim-quantseq-mrnaseq-se.cwl",
            "http://schema.org/codeRepository": "https://github.com/datirium/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Datirium, LLC",
                    "http://schema.org/member": [
                        {
                            "class": "http://schema.org/Person",
                            "http://schema.org/name": "Artem Barski",
                            "http://schema.org/email": "mailto:Artem.Barski@datirum.com"
                        },
                        {
                            "class": "http://schema.org/Person",
                            "http://schema.org/name": "Andrey Kartashov",
                            "http://schema.org/email": "mailto:Andrey.Kartashov@datirium.com",
                            "http://schema.org/sameAs": [
                                {
                                    "$import": "#0000-0001-9102-5681"
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    ],
    "cwlVersion": "v1.0",
    "$schemas": [
        "https://github.com/schemaorg/schemaorg/raw/main/data/releases/11.01/schemaorg-current-http.rdf"
    ],
    "$namespaces": {
        "s": "http://schema.org/"
    }
}
